"""@package realtime LBP

 verison 0.4.0
 Author : Petr Neduchal
 Date : 2013
"""

from PIL import Image
import ctypes
import numpy as np
import scipy

# Function loads dynamic library
def loadRealtimeLbpLibrary():
    """Function loads LBP library

			 libRealtimeLbp.dll/libRealtimeLbp.so (win/lin)
    """
    lbplib = ctypes.cdll.LoadLibrary("libRealtimeLbp.so")
    return lbplib


def realTimeLbp(lbplib, filename):
    """Function calls extern function realTimeLbp 

	     Params : lbplib - library pointer (see loadLbpLibrary)
                 filename - (string) path to the image file. 
    """
    im = Image.open(filename)
    w, h = im.size
    img = (ctypes.c_long * (w * h))()
    res = (ctypes.c_long * 256)()
    for i in range(w):
        for j in range(h):
            img[(w * i) + j] = im.getpixel((i, j))
    lbplib.realTimeLbp(w, h, ctypes.byref(img), ctypes.byref(res))
    return res


def realTimeLbpIm(lbplib, im):
    """Function calls extern function realTimeLbp 

	Params : lbplib - library pointer (see loadLbpLibrary)
                 im - Instance of Image class from PIL module
    """
    w, h = im.size
    img = (ctypes.c_long * (w * h))()
    res = (ctypes.c_long * 256)()
    for i in range(w):
        for j in range(h):
            img[(w * i) + j] = im.getpixel((i, j))
    lbplib.realTimeLbp(w, h, ctypes.byref(img), ctypes.byref(res))
    return res


def realTimeLbpImNp(lbplib, npIM):
    """Function calls extern function realTimeLbpImNp

	Params : lbplib - library pointer (see loadLbpLibrary)
                 npIM - NumPy matrix
    """
    w = npIM.shape[0]
    h = npIM.shape[1]
    img = (ctypes.c_long * (w * h))()
    res = (ctypes.c_long * 256)()
    res2 = np.zeros([1, 256])
    for i in range(w):
        for j in range(h):
            img[(w * i) + j] = npIM[i, j]
    lbplib.realTimeLbp(w, h, ctypes.byref(img), ctypes.byref(res))
    for i in range(256):
        res2[0, i] = res[i]
    return res2


def realTimeLbpIm2ImNp(lbplib, npIM):
    """Function calls extern function realTimeLbpImNp

	Params : lbplib - library pointer (see loadLbpLibrary)
                 npIM - NumPy matrix
    """
    w = npIM.shape[0]
    h = npIM.shape[1]
    img = (ctypes.c_long * (w * h))()
    res = (ctypes.c_long * (w * h))()
    for i in range(w):
        for j in range(h):
            img[(w * i) + j] = npIM[i, j]
    lbplib.realTimeLbpIm(w, h, ctypes.byref(img), ctypes.byref(res))
    for i in range(w):
        for j in range(h):
            res2[i, j] = res[(w * i) + j]
    return res2


def realTimeLbpArr(lbplib, data, w, h):
    """Function calls extern function realTimeLbp 

	Params : lbplib - library pointer (see loadLbpLibrary)
                 data - (ctypes.c_long * (size of the image)) C type array 
                 with Image data
                 w - image width
                 h - image height
    """
    res = (ctypes.c_long * 256)()
    lbplib.realTimeLbp(w, h, ctypes.byref(data), ctypes.byref(res))
    return res


def lbp2Hists(lbplib, IM):
    """Function calls extern function realTimeLbpImNp

	Params : lbplib - library pointer (see loadLbpLibrary)
                 npIM - NumPy matrix
    """
    w = npIM.shape[0]
    h = npIM.shape[1]
    img = (ctypes.c_long * (w * h))()
    res = (ctypes.c_long * (w * h))()
    for i in range(w):
        for j in range(h):
            img[(w * i) + j] = npIM[i, j]
    lbplib.lbp2Hists(w, h, ctypes.byref(img), ctypes.byref(res))
    return res


def lbp2HistsNp(lbplib, npIM):
    """Function calls extern function realTimeLbpImNp

	Params : lbplib - library pointer (see loadLbpLibrary)
                 npIM - NumPy matrix
    """
    w = npIM.shape[0]
    h = npIM.shape[1]
    img = (ctypes.c_long * (w * h))()
    res = (ctypes.c_long * (w * h))()
    for i in range(w):
        for j in range(h):
            img[(w * i) + j] = npIM[i, j]
    lbplib.lbp2Hists(w, h, ctypes.byref(img), ctypes.byref(res))
    for i in range(w):
        for j in range(h):
            res2[i, j] = res[(w * i) + j]
    return res2



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
VTK Viewer pro 3D zobrazeni

program mozne spustit ve dvou rezimech View a Cut. Pokud neni zvolen zadny mod, tak
prohlizec pracuje v rezimu View

Mod Cut :
Priklady spusteni:
viewer3.py -vtk mesh_new.vtk -mode 'Cut' -slab 'liver'
viewer3.py -pkl out -mode 'Cut'
viewer3.py -pkl out -mode 'Cut' -slab 'porta'

Spusti se editor volby resekcni linie. V nem je mozno volit rez pomoci roviny a to
stisknutim tlacitka Plane. Rez se provede stisknutim tlacitka Cut. Tlacitko
Point zatim nefunkcni

Mod View :
Priklady spusteni:
viewer3.py -pkl file.pkl -mode 'View'
viewer3.py -vtk mesh_new.vtk -mode 'View' -slab 'liver'
viewer3.py -pkl vessels002.pkl -mode 'View' -deg 5


Spusti prohlizec slouzici pouze pro vizualizaci jater

'''

'''
Importování potřebných knihoven a skriptů
'''
import sys
import virtual_resection
import numpy as np
import numpy as nm
import scipy.ndimage
import argparse

from PyQt4 import QtCore, QtGui
# from PyQt4.QtCore import pyqtSignal, QObject, QRunnable, QThreadPool, Qt
from PyQt4.QtGui import QMainWindow, QGridLayout, QApplication
# from PyQt4 import *
# from PyQt4.QtGui import *
# from PyQt4.QtCore import *

import vtk
from vtk.qt4.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from Tkinter import *
try:
    from dicom2fem import seg2fem
except:
    print "deprecated import of seg2fem"

    import seg2fem

import misc

'''
Používané globální proměnné
'''
plane = vtk.vtkPlane()
normal = None
coordinates = None
iren = vtk.vtkRenderWindowInteractor()
surface = vtk.vtkDataSetSurfaceFilter()
app = QApplication(sys.argv)
label = QtGui.QLabel()
myLayout = QGridLayout()
widget = vtk.vtkSphereSource()
planeWidget = vtk.vtkImplicitPlaneWidget()


'''
Kód grafického editoru resekční linie. Tento kód byl automaticky vygenerován pomocí programu Qt Designer
'''
##------------------------------------------------------------------------------------------
try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        '''
        Zde se vytvoří hlavní okno editoru. Nastaví se jeho velikost a název
        '''
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(1000, 800)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.widget = QtGui.QWidget(self.centralwidget)
        self.widget.setGeometry(QtCore.QRect(370, 49, 401, 411))
        self.widget.setObjectName(_fromUtf8("widget"))
        '''
        Vytvoření tlačítka CUT, které při stisku spouští metodu liver_cut (zvolený resekční algoritmus)
        '''
        self.toolButton = QtGui.QPushButton(self.centralwidget)
        self.toolButton.setGeometry(QtCore.QRect(140, 140, 71, 41))
        self.toolButton.setObjectName(_fromUtf8("toolButton"))
        QtCore.QObject.connect(self.toolButton, QtCore.SIGNAL("clicked()"), MainWindow.liver_cut )
        '''
        Vytvoření tlačítka PLANE, které při stisku volá metodu Plane
        '''
        self.toolButton_2 = QtGui.QPushButton(self.centralwidget)
        self.toolButton_2.setGeometry(QtCore.QRect(140, 280, 71, 41))
        self.toolButton_2.setObjectName(_fromUtf8("toolButton_2"))
        QtCore.QObject.connect(self.toolButton_2, QtCore.SIGNAL("clicked()"), MainWindow.Plane )
        '''
        Vytvoření tlačítka POINT, které při stisku volá metodu Point
        '''
        self.toolButton_3 = QtGui.QPushButton(self.centralwidget)
        self.toolButton_3.setGeometry(QtCore.QRect(140, 210, 71, 41))
        self.toolButton_3.setObjectName(_fromUtf8("toolButton_3"))
        QtCore.QObject.connect(self.toolButton_3, QtCore.SIGNAL("clicked()"), MainWindow.Point )
        '''
        Vytvoření textového pole pro uživatelské výpisy
        '''
        self.info_text = QtGui.QPlainTextEdit(self.centralwidget)
        self.info_text.setGeometry(QtCore.QRect(20, 350, 280, 100))
        self.info_text.setObjectName(_fromUtf8("lineEdit"))
        self.info_text.setReadOnly(True)
        '''
        Vytvoření textového pole pro výpisy informací o velikosti odstraněné části jater
        '''
        self.liver_text = QtGui.QPlainTextEdit(self.centralwidget)
        self.liver_text.setGeometry(QtCore.QRect(380, 490, 380, 50))
        self.liver_text.setObjectName(_fromUtf8("lineEdit"))
        self.liver_text.setReadOnly(True)

        '''
        Vytvoření vizualizačního okna
        '''
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 21))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)
        self.gridlayout = QtGui.QGridLayout(self.widget)
        self.vtkWidget = QVTKRenderWindowInteractor(self.widget)

        self.gridlayout.addWidget(self.vtkWidget, 0, 0, 1, 1)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(_translate("MainWindow", "Nástroj pro volbu resekční linie", None))
        self.toolButton.setText(_translate("MainWindow", "CUT", None))
        self.toolButton_2.setText(_translate("MainWindow", "PLANE", None))
        self.toolButton_3.setText(_translate("MainWindow", "POINT", None))
##------------------------------------------------------------------------------------------
'''
Hlavní třída pro vytvoření prohlížeče
'''
class Viewer(QMainWindow):

    '''
    Konstruktor pro vytvoření objektu prohlížeče
    Pokud je zvolen resekční režim, nejdříve se vytvoří editor resekční linie a poté se nastaví interaktor
    '''
    def __init__(self, inputfile,mode,parent = None):
        self.ren = vtk.vtkRenderer()
        if mode == 'Cut':
            QtGui.QMainWindow.__init__(self, parent)
            self.vtk_filename = inputfile
            self.ui = Ui_MainWindow()
            self.ui.setupUi(self)
            self.info_text = self.ui.info_text
            self.liver_text = self.ui.liver_text
            self.planew = None
            self.cut_point = None
            self.iren = self.ui.vtkWidget.GetRenderWindow().GetInteractor()
            self.ui.vtkWidget.GetRenderWindow().AddRenderer(self.ren)
        '''
        Pokud je zvolen prohlížecí režim, musíme vytvořit jak renderovací okna tak samotný interaktor
        '''
        if mode == 'View':
            QMainWindow.__init__(self,parent)
            self.renWin = vtk.vtkRenderWindow()
            self.renWin.AddRenderer(self.ren)
            self.iren = vtk.vtkRenderWindowInteractor()
            self.iren.SetRenderWindow(self.renWin)

##------------------------------------------------------------------------------------------
    '''
    Tato metoda slouží pro vygenerování vtk souboru ze segmentovaných dat. Této metodě jsou tedy
    předána segmentovaná data společně s velikostí voxelu.
    '''
    def generate_mesh(self,segmentation,voxelsize_mm):
        self.segment = segmentation
        print("Generuji data...")
        self.voxelsize_mm = voxelsize_mm
        '''
        Nahrazuje původní velikost voxelu, velikostí vexoelů zvětšenou o hodnotu degradace
        '''
        self.new_vox = voxelsize_mm * self.degrad
        '''
        Zde je použita metoda ze skriptu seg2fem, která ze zadaných dat a velikosti voxelu vytvoří vtk soubor
        Data se před vytvořením vtk souboru ještě vyhlazují,aby měla hladší povrch
        Výsledný soubor je uložen pod názvem mesh_new.vtk
        '''
        mesh_data = seg2fem.gen_mesh_from_voxels_mc(self.segment, self.new_vox)
        if True:
            for x in xrange (50):
                mesh_data.coors = seg2fem.smooth_mesh(mesh_data)

        print("Done")
        vtk_file = "mesh_new.vtk"
        mesh_data.write(vtk_file)
        return vtk_file

##------------------------------------------------------------------------------------------
    '''
    Tato metoda slouží pro vytvoření virtuální roviny. K tomu je využita třída z VTK vtkImlicitPlaneWidget()
    '''
    def Plane(self):
        if(self.planew != None):
            self.info_text.appendPlainText (_fromUtf8("Nelze použít více rovin najednou. Nejdříve proveďte řez"))
        else:
            planeWidget = vtk.vtkImplicitPlaneWidget()
            # předaní interaktoru objektu roviny
            planeWidget.SetInteractor(self.iren)
            # nastavení velikosti prostoru ve kterém se může rovina pohybovat
            planeWidget.SetPlaceFactor(1.5)
            # nastavení vstupních dat
            planeWidget.SetInput(surface.GetOutput())
            planeWidget.PlaceWidget()
            planeWidget.TubingOff()
            planeWidget.OutsideBoundsOff()
            planeWidget.ScaleEnabledOff()
            planeWidget.OutlineTranslationOff()
            planeWidget.AddObserver("InteractionEvent", self.Cutter)

            planeWidget.On()
            self.planew = planeWidget
            self.planew.SetNormal(2.0,0.0,0.0)

##------------------------------------------------------------------------------------------
    '''
    Tato metoda slouží pro vytvoření virtuálního bodu. K tomu je využita třída z VTK vtkPointWidget()
    '''
    def Point(self):
            print 'Point'
            self.cut_point = vtk.vtkPointWidget()
            self.cut_point.SetInput(surface.GetOutput())
            self.cut_point.AllOff()
            self.cut_point.PlaceWidget()
            # nastavení interaktoru
            self.cut_point.SetInteractor(self.iren)
            self.cut_point.On()
            point = vtk.vtkPolyData()
            self.cut_point.GetPolyData(point);

##------------------------------------------------------------------------------------------

    def Cutter(self,obj, event):
        global plane, selectActor
        obj.GetPlane(plane)

    '''
    Tato metoda obsluhuje spouštění resekčních algoritmů.
    '''
    def liver_cut(self):
        '''
        Pokud není zvolené kriterium pro provádění resekční linie, program zahlásí chybu
        '''
        if (self.planew == None) & (self.cut_point == None):
            self.info_text.appendPlainText (_fromUtf8("Neexistuje rovina řezu"))
            self.info_text.appendPlainText (_fromUtf8('Nejdříve vytvořte rovinu(Plane), nebo bod(Point)'))
        '''
        Pokud je zvoleno jako resekční kritérium rovina spustí se metoda Rez_podle_roviny ze skriptu virtual_resection
        '''
        if self.planew != None:
            self.info_text.appendPlainText (_fromUtf8("Provádění řezu. Prosím čekejte"))
            data_z_resekce,odstraneni_procenta = virtual_resection.Rez_podle_roviny(self.planew,self.segment,self.new_vox)
            '''
            Zde se provádí výpis velikosti (v procentech) odříznuté části jater do editoru pro uživatele
            Pokud je z jater odříznuto příliš mnoho, algoritmus někdy přepočítá jejich hodnotu nad hranici sta procent, proto
            je to zde ošetřeno podmínkou
            '''
            if (odstraneni_procenta > 100):
                self.liver_text.appendPlainText(_fromUtf8("Odstraněno příliš mnoho. Nelze spočítat"))
            else:
                self.liver_text.appendPlainText(_fromUtf8("Bylo ostraněno cca "+str(odstraneni_procenta)+" % jater"))
            '''
            Zde je vypnuta vytvořená rovina, což je provedeno z toho důvodu, aby mohlo být prováděno více řezů za sebou
            '''
            self.planew.Off()
        '''
        Pokud je zvoleno jako resekční kritérium bod spustí metoda. Tato metoda nejdříve vytvoří metici nul o stejné velikosti jako je matice
        původních dat. V této matici je na pozici bodu, který zvolí uživatel nula naahrazena jedničkou. Celá tato matice je společně s maticí
        původních dat předána resekčnímu algoritmu podle bodu ze skriptu virtual_resection
        '''
        if self.cut_point != None:
            self.info_text.appendPlainText (_fromUtf8("Provádění řezu. Prosím čekejte"))
            pozice = self.cut_point.GetPosition()
            self.data['data3d'] = self.data['data3d'][::self.degrad,::self.degrad,::self.degrad]
            seeds = np.zeros((self.data['segmentation'].shape[0],(self.data['segmentation'].shape[1]),(self.data['segmentation'].shape[2])))
            seeds[pozice[0]/self.new_vox[0]][pozice[1]/self.new_vox[1]][pozice[2]/self.new_vox[2]] = 1
            self.data = virtual_resection.Resekce_podle_bodu(self.data,seeds)
            data_z_resekce = self.data['segmentation'] == self.data['slab']['liver']
            self.cut_point.Off()
        '''
        Následně je vytvořen soubor VTK obsahující část jater, bez uříznuté části.
        '''
        mesh_data = seg2fem.gen_mesh_from_voxels_mc(data_z_resekce, self.new_vox)
        if True:
            for x in xrange (15):
                mesh_data.coors = seg2fem.smooth_mesh(mesh_data)
        print("Done")
        self.planew = None
        self.cut_point = None
        vtk_file = "mesh_new.vtk"
        mesh_data.write(vtk_file)
        self.View(vtk_file)


##------------------------------------------------------------------------------------------
    '''
    Tato metoda slouží pro práci s editorem a prohlížečem v jiném programu bez použítí příkazové řádky (Zatím ve zkušebním provozu, ještě není plně funkční!)
    '''
    def prohlizej(self,data, mode, slab=None):

        if slab == 'liver':
            self.degrad = 5
        else:
            self.degrad = 5
        mesh = self.generate_mesh(data['segmentation'] == data['slab'][slab],data['voxelsize_mm'])
        #if mode == 'View' or mode == None:
        #if mode == 'Cut':
        self.View(mesh)
        self.iren.Initialize()
        app.exec_()
        sys.exit(app.exec_())


        return self


##-----------------------------------------------------------------------------------------
    '''
    Tato metoda slouží pro vizualizaci dat pomocí knihoven a tříd z VTK
    '''

    def View(self,filename):

        '''
        Nastavení interaktoru pro pohyb s objektem. Třída vtkInteractorStyleTrackballCamera(), nám umožní nastavit na levé tlačítko
        myši funkce pohybu s vizualizovaným objektem a na pravé možnost zoomu
        '''
        self.iren.SetInteractorStyle(vtk.vtkInteractorStyleTrackballCamera())

        '''
        Nastavení readeru pro čtení vtk souboru v podobě nestrukturované mřížky
        '''
        reader = vtk.vtkUnstructuredGridReader()
        reader.SetFileName(filename)
        reader.Update()

        '''
        Jako filtr je použit objekt třídy vtkDataSetSurfaceFilter(), který nám z dat vyextrahuje vnější povrch
        '''
        surface.SetInput(reader.GetOutput())
        surface.Update()

        '''
        Dále použijeme třídu vtkClipPolyData(), ta nám při pohybu roviny způsobí, že za ní bude nechávat pouze obrysy spojení
        buněk bez vyplnění povrchu, tím snadno poznáme kde jsme s rovinou po objektu již přejeli a kde ne
        '''
        clipper = vtk.vtkClipPolyData()
        clipper.SetInput(surface.GetOutput())
        clipper.SetClipFunction(plane)
        clipper.GenerateClippedOutputOn()

        clipMapper = vtk.vtkPolyDataMapper()
        clipMapper.SetInput(clipper.GetOutput())

        clipActor = vtk.vtkActor()
        clipActor.SetMapper(clipMapper)

        mapper = vtk.vtkDataSetMapper()
        mapper.SetInput(surface.GetOutput())



        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        actor.GetProperty().EdgeVisibilityOn()
        # nastavuje šířku linek ohraničující buňky
        actor.GetProperty().SetLineWidth(0.1)
        actor.GetProperty().SetRepresentationToWireframe()
        self.ren.AddActor(clipActor)
        self.ren.AddActor(actor)

        self.iren.Initialize()
        self.iren.Start()
        '''
        Třídu vtkWindowToImageFilter použijeme pro uložení vizualizace z prohlížecího režimu ve formátu tif
        '''
        try:
            w2i = vtk.vtkWindowToImageFilter()
            writer = vtk.vtkTIFFWriter()
            w2i.SetInput(self.renWin)
            w2i.Update()
            writer.SetInputConnection(w2i.GetOutputPort())
            writer.SetFileName("image.tif")
            self.renWin.Render()
            writer.Write()
            self.renWin.Render()
            self.renWin.Finalize()
        except(AttributeError):
            print()


##------------------------------------------------------------------------------------------
def main():
    '''
    Parser slouží pro zadávání vstupních parametrů přes příkazovou řádku. Je možné zadat celkem 6 parametrů.
    K čemu jednotlivé parametry slouží, se můžeme dočíst při zadání příkazu (viewer3.py --help) do příkazové řádky
    '''

    parser = argparse.ArgumentParser(description='Simple VTK Viewer')

    parser.add_argument('-pkl','--picklefile', default=None,
                      help='Zadání vstupního zdroje dat. Soubor .pkl jsou zpravidla segmentovaná data')
    parser.add_argument('-vtk','--vtkfile', default=None,
                      help='Zadání vstupního zdroje dat. Soubor .vtk je soubor vygenerovaný programem VTK')
    parser.add_argument('-mode','--mode', default='View',
                      help='Zadání resekčního, nebo zobrazovacího režimu')
    parser.add_argument('-slab','--slab', default = 'liver',
                      help='Zde zadáváme zda chceme zobrazit játra, nebo portální žílu')
    parser.add_argument('-vs','--voxelsize_mm', default = [1,1,1],
                      type=eval,
                      help='Viewer_size')
    parser.add_argument('-deg','--degradace', default = None, type=int,
                      help='Hodnota degradace (snizeni poctu dat)')

    args = parser.parse_args()

    '''
    Pokud programu nezadáme data vyhlásí chybu, že nemá data
    '''
    if (args.picklefile or args.vtkfile) is None:
       raise IOError('No input data!')

    '''
    Zde se program větví na dvě možnosti podle toho, jaký druh zdroje dat jsme zvolili
    Pokud jsme zvolili jako zdroj dat pickle soubor (segmentovaná data) vytvoří se objekt
    prohlížeče (Viewer) a předá se mu tento zdro, společně s informací zda chceme program
    spustit v resekčním, nebo prohlížecím režimu.
    '''
    if args.picklefile:
        viewer = Viewer(args.picklefile,args.mode)
        data = misc.obj_from_file(args.picklefile, filetype = 'pickle')

        '''
        Zde programu zadáme hodnotu degradace (pokud není změněna parametrem deg)
        '''
        if (args.slab == 'porta') &  (args.degradace is None):
            viewer.degrad = 2
        if (args.slab == 'liver') &  (args.degradace is None):
            viewer.degrad = 4
        if args.degradace != None:
            viewer.degrad = args.degradace

        '''
        Data jsou zmenšována degradací v každém rozměru
        '''
        viewer.data = data
        viewer.data['segmentation'] = viewer.data['segmentation'][::viewer.degrad,::viewer.degrad,::viewer.degrad]
        viewer.data['segmentation'] = viewer.data['segmentation'][:,::-1,:]
        '''
        Pokud data neobsahují portální žílu je zahlášena chyba a program dále pracuje s celými játry
        '''
        try:
            mesh = viewer.generate_mesh(viewer.data['segmentation'] == viewer.data['slab'][args.slab],viewer.data['voxelsize_mm'])
        except KeyError:
            try:
                print 'Data bohuzel neobsahuji zadany slab:', args.slab
                print 'Zobrazena budou pouze dostupna data'
                viewer.info_text.appendPlainText (_fromUtf8('Data bohužel neobsahují zadanou část jater'))
                viewer.info_text.appendPlainText (_fromUtf8('Zobrazena budou pouze dostupná data'))
                mesh = viewer.generate_mesh(viewer.data['segmentation'] == viewer.data['slab']['liver'],viewer.data['voxelsize_mm'])
                '''
                Pokud data navíc neobsahují parametr rozměr voxelu (velmi vyjímečná záležitost) jsou programu předány jednotkové
                rozměry voxelu (1,1,1)
                '''
            except KeyError:
                data['voxelsize_mm'] = np.squeeze([1,1,1])
                mesh = viewer.generate_mesh(viewer.data['segmentation'] == viewer.data['slab'][args.slab],viewer.data['voxelsize_mm'])

        '''
        Pokud je zadán mód Cut(prohlížecí režim) nejdříve zobrazíme editor, abychom mohli začít vypisovat
        informace pro uživatele
        '''
        if args.mode == 'Cut':
            viewer.show()
        viewer.View(mesh)

    '''
    Pokud je zadán jako vstupní zdroj dat soubor vtk. Je vytvořen objekt prohlížeče(Viewer) a je mu předán vstupní soubor
    společně se zvoleným módem
    '''
    if args.vtkfile:
        viewer = Viewer(args.vtkfile,args.mode)
        if args.mode == 'Cut':
            viewer.show()
        viewer.View(args.vtkfile)

    viewer.iren.Initialize()
    app.exec_()
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()

##------------------------------------------------------------------------------------------




#! /usr/bin/python
# -*- coding: utf-8 -*-

"""
Generator of histology report

"""
import logging
logger = logging.getLogger(__name__)


# import funkcí z jiného adresáře
import sys
import os.path


path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../../lar-cc/lib/py/"))
sys.path.append(os.path.join(path_to_script, "../lisa/extern"))
sys.path.append(os.path.join(path_to_script, "../../pyplasm/src/pyplasm"))

import numpy as np

from scipy import mat, cos, sin

from larcc import VIEW, MKPOL, AA, INTERVALS, STRUCT, MAP, PROD
from larcc import UNITVECT, VECTPROD, PI, SUM, CAT, IDNT, UNITVECT
from splines import BEZIER, S1, S2, COONSPATCH
# from splines import *
# import mapper
#import hpc
#import pyplasm.hpc


import geometry3d as g3
import interpolation_pyplasm as ip


class GTLarSmooth:

    def __init__(self, gtree=None):
        """
        gtree is information about input data structure.
        endDistMultiplicator: make cylinder shorter by multiplication of radius
        """
# input of geometry and topology
        self.V = []
        self.CV = []
        self.joints = {}
        self.joints_lar = []
        self.gtree = gtree
        self.endDistMultiplicator = 2
        self.use_joints = True
        #dir(splines)
        pass

    def add_cylinder(self, nodeA, nodeB, radius, cylinder_id):

        try:
            idA = tuple(nodeA)  # self.gtree.tree_data[cylinder_id]['nodeIdA']
            idB = tuple(nodeB)  # self.gtree.tree_data[cylinder_id]['nodeIdB']
        except:
            idA = 0
            idB = 0
            self.use_joints = False

        vect = np.array(nodeA) - np.array(nodeB)
        u = g3.perpendicular_vector(vect)
        u = u / np.linalg.norm(u)
        u = u.tolist()
        vect = vect.tolist()



        c1 = self.__circle(nodeA, radius, vect)
        c2 = self.__circle(nodeB, radius, vect)
        tube = BEZIER(S2)([c1,c2])
        domain = PROD([ INTERVALS(2*PI)(36), INTERVALS(1)(4) ])
        tube = MAP(tube)(domain)


        self.joints_lar.append(tube)


        #self.__draw_circle(nodeB, vect, radius)

        ##vector = (np.array(nodeA) - np.array(nodeB)).tolist()

# mov circles to center of cylinder by size of radius because of joint
        ##nodeA = g3.translate(nodeA, vector,
        ##                     -radius * self.endDistMultiplicator)
        ##nodeB = g3.translate(nodeB, vector,
        ##                     radius * self.endDistMultiplicator)

        ##ptsA, ptsB = g3.cylinder_circles(nodeA, nodeB, radius, element_number=32)
        ##CVlistA = self.__construct_cylinder_end(ptsA, idA, nodeA)
        ##CVlistB = self.__construct_cylinder_end(ptsB, idB, nodeB)

        ##CVlist = CVlistA + CVlistB

        ##self.CV.append(CVlist)

# lar add ball
#         ball0 = mapper.larBall(radius, angle1=PI, angle2=2*PI)([10, 16])
#         V, CV = ball0
#         # mapper.T
#         # ball = STRUCT(MKPOLS(ball0))
#
#         # mapper.T(1)(nodeA[0])(mapper.T(2)(nodeA[1])(mapper.T(3)(nodeA[1])(ball)))
#
#         lenV = len(self.V)
#
#         self.V = self.V + (np.array(V) + np.array(nodeA)).tolist()
#         self.CV = self.CV + (np.array(CV) + lenV).tolist()

    def __circle(self, center=[0,0,0],radius=1,normal=[0,0,1],sign=1,shift=0):
        N = UNITVECT(normal)
        if N == [0,0,1] or N == [0,0,-1]: Q = mat(IDNT(3))
        else: 
            QX = UNITVECT((VECTPROD([[0,0,1],N])))
            QZ = N
            QY = VECTPROD([QZ,QX])
            Q = mat([QX,QY,QZ]).T
        def circle0(p):
            u = p[0]
            x = radius*cos(sign*u+shift)
            y = radius*sin(sign*u+shift)
            z = 0
            return SUM([ center, CAT((Q*[[x],[y],[z]]).tolist()) ])
        return circle0


    def __construct_cylinder_end(self, pts, id, node):
        """
        creates end of cylinder and prepares for joints
        """
        CVlist = []
        # base
        ln = len(self.V)

        for i, pt in enumerate(pts):
            self.V.append(pt)
            CVlist.append(ln + i)

        try:
            self.joints[id].append([node, CVlist])
        except:
            self.joints[id] = [[node, CVlist]]

        return CVlist

    def __add_old_cylinder(self, nodeA, nodeB, radius):
        """
        deprecated simple representation of cylinder
        """
        nodeA = np.array(nodeA)
        nodeB = np.array(nodeB)

        ln = len(self.V)
        self.V.append(nodeB.tolist())
        self.V.append((nodeB + [2, 0, 0]).tolist())
        self.V.append((nodeB + [2, 2, 0]).tolist())
        self.V.append((nodeB + [2, 2, 2]).tolist())
        self.V.append((nodeA + [0, 0, 0]).tolist())
        self.CV.append([ln, ln + 1, ln + 2, ln + 3, ln + 4])

    def finish(self):
        print 'use joints? ', self.use_joints
        if self.use_joints:
            for joint in self.joints.values():
                # There is more then just one circle in this joint, so it
                # is not end of vessel
                if len(joint) > 1:
                    self.__generate_joint(joint)


    def __half_plane(self, perp, plane_point, point):
        cdf = (np.array(point) - np.array(plane_point))
        out = perp[0] * cdf[0] +\
            perp[1] * cdf[1] + \
            perp[2] * cdf[2]
        return  out > 0

    def __get_vessel_connection_curve(self, vessel_connection, perp, vec0, vec1):
        """
        perp is perpendicular to plane given by centers of circles
        vec1, vec0 are vectors from circle centers
        """
        curve_t = []
        curve_d = []
        curve_pts_indexes_t = []
        curve_pts_indexes_d = []
        brake_point_t = None
        brake_point_d = None
        center, circle = vessel_connection

        # left to right
        perp_lr = np.cross(perp, vec1)

        print 'center ', center
        print 'circle ', circle
        for vertex_id in circle:
            if ((len(curve_pts_indexes_t) > 0) and 
                (vertex_id - curve_pts_indexes_t[-1]) > 1):
                brake_point_t = len(curve_pts_indexes_t)
            if ((len(curve_pts_indexes_d) > 0) and 
                (vertex_id - curve_pts_indexes_d[-1]) > 1):
                brake_point_d = len(curve_pts_indexes_d)

            #hp = self.__half_plane(perp_lr, center, self.V[vertex_id])
            hp = self.__half_plane(perp, center, self.V[vertex_id])

            
            if(hp):
                curve_t.append(self.V[vertex_id])
                curve_pts_indexes_t.append(vertex_id)
            else:
                curve_d.append(self.V[vertex_id])
                curve_pts_indexes_d.append(vertex_id)

        ordered_curve_t = curve_t[brake_point_t:] + curve_t[:brake_point_t]
        ordered_pts_indexes_t = \
            curve_pts_indexes_t[brake_point_t:] +\
            curve_pts_indexes_t[:brake_point_t]

        ordered_curve_d = curve_d[brake_point_d:] + curve_d[:brake_point_d]
        ordered_pts_indexes_d = \
            curve_pts_indexes_d[brake_point_t:] +\
            curve_pts_indexes_d[:brake_point_d]
        #print '    hp v id ', curve_pts_indexes_t    
        #print 'ord hp v id ', ordered_pts_indexes_t

        #print 'hp circle ', curve_one

        # add point from oposit half-circle
        first_pt_d = ordered_curve_d[0]
        last_pt_d = ordered_curve_d[-1]
        first_pt_t = ordered_curve_t[0]
        last_pt_t = ordered_curve_t[-1]

        ordered_curve_t.append(first_pt_d)
        ordered_curve_t.insert(0, last_pt_d)

        ordered_curve_d.append(first_pt_t)
        ordered_curve_d.insert(0, last_pt_t)

        return ordered_curve_t, ordered_curve_d

    def __generate_joint(self, joint):
        #joint = (np.array(joint).reshape(-1)).tolist()
        #self.CV.append(joint)
        cc0 = np.array(joint[0][0])
        cc1 = np.array(joint[1][0])
        cc2 = np.array(joint[2][0])

        vec0 = cc0 - cc1
        vec1 = cc1 - cc2

        perp = np.cross(vec0, vec1)


        curvelistT = []
        curvelistD = []

        for vessel_connection in joint:
            ordered_curve_t, ordered_curve_d = self.__get_vessel_connection_curve(
                vessel_connection, perp, vec0, vec1)
            


            curvelistT.append(ordered_curve_t)
            curvelistD.append(ordered_curve_d)
                #print '  ', self.V[vertex_id], '  hp: ', hp

        Betacurve_id, Astart, Alphacurve_id, Bstart, Gammacurve_id, Cstart = self.__find_couples(curvelistT)
        
        #print 'ABC ', Betacurve_id, Astart, Alphacurve_id, Bstart

        dom2D = ip.TRIANGLE_DOMAIN(32, [[1,0,0],[0,1,0],[0,0,1]])
        Cab0 = BEZIER(S1)(self.__order_curve(curvelistT[Gammacurve_id][-1:0:-1], Cstart))
        Cbc0 = BEZIER(S1)(self.__order_curve(curvelistT[Alphacurve_id], Bstart))
        Cbc1 = BEZIER(S2)(self.__order_curve(curvelistT[Alphacurve_id], Bstart))
        Cca0 = BEZIER(S1)(self.__order_curve(curvelistT[Betacurve_id][-1:0:-1], Astart))
        
        out1 = MAP(ip.TRIANGULAR_COONS_PATCH([Cab0,Cbc1,Cca0]))(STRUCT(dom2D))
        self.joints_lar.append(out1)

        Betacurve_id, Astart, Alphacurve_id, Bstart, Gammacurve_id, Cstart = self.__find_couples(curvelistD)
        
        #print 'ABC ', Betacurve_id, Astart, Alphacurve_id, Bstart

        dom2D = ip.TRIANGLE_DOMAIN(32, [[1,0,0],[0,1,0],[0,0,1]])
        Cab0 = BEZIER(S1)(self.__order_curve(curvelistD[Gammacurve_id][-1:0:-1], Cstart))
        Cbc0 = BEZIER(S1)(self.__order_curve(curvelistD[Alphacurve_id], Bstart))
        Cbc1 = BEZIER(S2)(self.__order_curve(curvelistD[Alphacurve_id], Bstart))
        Cca0 = BEZIER(S1)(self.__order_curve(curvelistD[Betacurve_id][-1:0:-1], Astart))

        out2 = MAP(ip.TRIANGULAR_COONS_PATCH([Cab0,Cbc1,Cca0]))(STRUCT(dom2D))
        self.joints_lar.append(out2)

    def __find_couples(self, curvelist):
        """
        try find all posible couples with minimal energy. 
        Energy is defined like sum of distances
        """
        energy = None
        mn_ind = None
        output = None
        for i in range(0,3):
            Betacurve_id, Astart, dist0 = self.__find_nearest(
                curvelist, i, 0, [i])
            Alphacurve_id, Bstart, dist1 = self.__find_nearest(
                curvelist, i, -1, [i, Betacurve_id])
            this_energy = dist0 + dist1

            if energy is None or this_energy < energy:
                energy = this_energy
                mn_ind = i
                #Gammacurve_id = i
                output = Betacurve_id, Astart, Alphacurve_id, Bstart, i, 0


            Betacurve_id, Astart, dist0 = self.__find_nearest(
                curvelist, i, -1, [i])
            Alphacurve_id, Bstart, dist1 = self.__find_nearest(
                curvelist, i, 0, [i, Betacurve_id])
            this_energy = dist0 + dist1

            if energy is None or this_energy < energy:
                energy = this_energy
                mn_ind = i
                output = Betacurve_id, Astart, Alphacurve_id, Bstart, i, -1

        print 'output'
        print output

        return output


    def __order_curve(self, curve, start):
        if start is 0:
            return curve
        else:
            return curve[-1:0:-1]

    def __find_nearest(self, curvelist, this_curve_index, start, wrong_curve=None):
        """
        start: use 0 or -1
        """
        #if start:
        #    start_index = 0
        #else:
        #    start_index = -1
        if wrong_curve is None:
            wrong_curve = [this_curve_index]
        dist = None
        min_cv_ind = None
        min_cv_start = None

        for curve_index in range(0, len(curvelist)):
            if curve_index not in wrong_curve:
                pt0 = np.array(curvelist[this_curve_index][start])
                pt1 = np.array(curvelist[curve_index][0])
                this_dist = np.linalg.norm(pt0 - pt1)
                if (dist is None) or (this_dist < dist):
                    dist = this_dist
                    min_cv_ind = curve_index
                    min_cv_start = 0

                pt1 = np.array(curvelist[curve_index][-1])
                this_dist = np.linalg.norm(pt0 - pt1)
                if (dist is None) or (this_dist < dist):
                    dist = this_dist
                    min_cv_ind = curve_index
                    min_cv_start = -1

        return min_cv_ind, min_cv_start, dist





    def show(self):

        V = self.V
        CV = self.CV

        # V = [[0,0,0],[5,5,1],[0,5,5],[5,5,5]]
        # CV = [[0,1,2,3]]
        # print 'V, CV ', V, CV
        #for joint in self.joints_lar:

        # out = STRUCT([MKPOL([V, AA(AA(lambda k:k + 1))(CV), []])] + self.joints_lar)
        out = STRUCT(self.joints_lar)
        #VIEW(self.joints_lar[0])
        #VIEW(MKPOL([V, AA(AA(lambda k:k + 1))(CV), []]))
        VIEW(out)
    def get_output(self):
        pass

    def __add_tetr(self, nodeB):
        """
        Creates tetrahedron in specified position.
        """
        try:
            nodeB = nodeB.tolist()
        except:
            pass

        ln = len(self.V)
        self.V.append(nodeB)
        self.V.append((np.array(nodeB) + [2, 0, 0]).tolist())
        self.V.append((np.array(nodeB) + [2, 2, 0]).tolist())
        self.V.append((np.array(nodeB) + [2, 2, 2]).tolist())
        self.CV.append([ln, ln + 1, ln + 2, ln + 3])

    def __add_cone(self, nodeA, nodeB, radius):
        vect = (np.array(nodeA) - np.array(nodeB)).tolist()
        pts = self.__circle(nodeA, vect, radius)

        ln = len(self.V)
        self.V.append(nodeB)
        # first object is top of cone
        CVlist = [ln]

        for i, pt in enumerate(pts):
            self.V.append(pt)
            CVlist.append(ln + i + 1)

        self.CV.append(CVlist)

    def __add_circle(self, center, perp_vect, radius, polygon_element_number=10):
        """
        Draw circle some circle points as tetrahedrons.
        """
        pts = g3.circle(center, perp_vect, radius,
                        polygon_element_number=polygon_element_number)
        for pt in pts:
            self.__add_tetr(pt)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 mjirik <mjirik@mjirik-Latitude-E6520>
#
# Distributed under terms of the MIT license.

"""

"""
import numpy as np

import logging
logger = logging.getLogger(__name__)
import argparse

import qmisc
from scipy import ndimage
'''SMAZAT'''

class ShapeModel():
    """
    Cílem je dát dohromady vstupní data s různou velikostí a různou polohou
    objektu. Výstup je pak zapotřebí opět přizpůsobit libovolné velikosti a
    poloze objektu v obraze.

    Model je tvořen polem s velikostí definovanou v konstruktoru (self.shape).
    U modelu je potřeba brát v potaz polohu objektu. Ta je udávána pomocí
    crinfo. To je skupina polí s minimální a maximální hodnotou pro každou osu.

    Trénování je prováděno opakovaným voláním funkce train_one().

    :param model_margin: stanovuje velikost okraje v modelu. Objekt bude ve
    výchozím nastavení vzdálen 0 px od každého okraje.

    """

    def __init__(self, shape=[5, 5, 5]):
        """TODO: to be defined1. """
        self.model = np.ones(shape)
        self.data_number = 0
        self.model_margin = [0, 0, 0]
        pass

    def get_model(self, crinfo, image_shape):
        """
        :param image_shape: Size of output image
        :param crinfo: Array with min and max index of object for each axis.
        [[minx, maxx], [miny, maxy], [minz, maxz]]

        """
        # Průměrování
        mdl = self.model / self.data_number
        print mdl.shape
        print crinfo
        # mdl_res = qmisc.resize_to_shape(mdl, crinfo[0][]
        uncr = qmisc.uncrop(mdl, crinfo, image_shape, resize=True)
        return uncr

    def train_one(self, data,voxelSize_mm):
        """
        Trenovani shape modelu
        data se vezmou a oriznou (jen jatra)
        na oriznuta data je aplikovo binarni otevreni - rychlejsi nez morphsnakes
        co vznikne je uhlazena cast ktera se odecte od puvodniho obrazu
        cimz vzniknou spicky
        orezany obraz se nasledne rozparceluje podle velikosti (shape) modelu
        pokud pocet voxelu v danem useku prekroci danou mez, je modelu 
        prirazena nejaka hodnota. Meze jsou nasledujici:
        0%-50% => 1
        50%-75% => 2
        75%-100% => 3
        """
        
        crinfo = qmisc.crinfo_from_specific_data(data, margin=self.model_margin)
        datacr = qmisc.crop(data, crinfo=crinfo)
        dataShape = self.model.shape
        datacrres = self.trainThresholdMap(datacr, voxelSize_mm, dataShape)

        self.model += datacrres

        self.data_number += 1

# Tady bude super kód pro trénování

    def train(self, data_arr):
        for data in data_arr:
            self.train_one(data)
            
    def objectThreshold(self,objekt,thresholds,values):
        '''
        Objekt - 3d T/F pole 
        thresholds = [0,0.5,0.75] zacina nulou
        values = [3,2,1]
        vrati hodnotu z values odpovidajici thresholds
        podle podilu True voxelu obsazenych v 3d poli 
        zde napriklad 60% =>2, 80% => 1.
        '''
        bile = np.sum(objekt)
        velikost = objekt.shape
        velikostCelkem = 1.0
        for x in velikost:
            velikostCelkem = velikostCelkem*x
            
        podil = bile/velikostCelkem #podil True voxelu    
        #print podil    
        #vybrani threshold
        final = 0 #vracena hodnota
        pomocny = 0 #pomocna promenna
        for threshold in thresholds:
            if(podil >= threshold ):
                final = values[pomocny]
            pomocny = pomocny+1
        return final
        
    
    def rozdelData(self,crData,dataShape, nasobitel1=1,nasobitel2 = 2):
        '''
        crData - vstupni data
        dataShape - velikost vraceneho pole
        volte 0<nasobitel1 < nasobitel2, vysvetleni nasleduje:
        rozdeli pole crData na casti vrati pole rozmeru dataShape
        vysledne hodnoty pole jsou urceny funkci objectThreshold(object,thresholds,values)
        intervaly prirazeni values [1-3] jsou nasledujici: 
        [0-prumer*nasobitel1],[prumer*nasobitel1-prumer*nasobitel2],[prumer*nasobitel2 a vice]
        '''
            
        'vypocet prumerneho podilu bilych voxelu'
        bile = np.sum(crData)
        velikost = crData.shape
        velikostCelkem = 1.0
        for x in velikost:
            velikostCelkem = velikostCelkem*x
            
        podil = bile/velikostCelkem #prumerny podil True voxelu
        
        thresholds = [0,nasobitel1*podil,nasobitel2*podil]
        values = [3,2,1]  
        
        'vybrani voxelu a vytvoreni objektu'    
        velikostDat = crData.shape
        voxelySmer = [0,0,0]
        vysledek = np.zeros(dataShape)
        for poradi in range(3):
            voxelySmer[poradi] = velikostDat[poradi]/dataShape[poradi]   
        
        for x in range(dataShape[0]):
            for y in range(dataShape[1]):
                for z in range(dataShape[2]):
                    xStart = x*voxelySmer[0]
                    xKonec = xStart+voxelySmer[0]
                    
                    yStart = y*voxelySmer[1]
                    yKonec = yStart+voxelySmer[1]
                    
                    zStart = z*voxelySmer[2]
                    zKonec = zStart+voxelySmer[2]
                    objekt = crData[xStart:xKonec,yStart:yKonec,zStart:zKonec]
                    vysledek[x,y,z] = self.objectThreshold(objekt,thresholds,values)
                    
                    
        return vysledek
    
    def vytvorKouli3D(self,voxelSize_mm,polomer_mm):
        '''voxelSize:mm = [x,y,z], polomer_mm = r
        Vytvari kouli v 3d prostoru postupnym vytvarenim
        kruznic podel X (prvni) osy. Predpokladem spravnosti
        funkce je ze Y a Z osy maji stejne rozliseni
        funkce vyuziva pythagorovu vetu'''
    
        print 'zahajeno vytvareni 3D objektu'
    
        x = voxelSize_mm[0]
        y = voxelSize_mm[1]
        z = voxelSize_mm[2]
        xVoxely = int(np.ceil(polomer_mm/x))
        yVoxely = int(np.ceil(polomer_mm/y))
        zVoxely = int( np.ceil(polomer_mm/z))
    
        rozmery = [xVoxely*2+1,yVoxely*2+1,yVoxely*2+1]
        xStred  = xVoxely
        konec = yVoxely*2+1
        koule = np.zeros(rozmery) #pole kam bude ulozen vysledek
    
        for xR in range(xVoxely*2+1):
    
            if(xR == xStred):
                print '3D objekt z 50% vytvoren'
    
            c = polomer_mm #nejdelsi strana
            a = (xStred-xR )*x
            vnitrek = (c**2-a**2)
            b = 0.0
            if(vnitrek > 0):
                b = np.sqrt((c**2-a**2))#pythagorova veta   b je v mm
            rKruznice = float(b)/float(y)
            if(rKruznice == np.NAN):
                continue
            #print rKruznice #osetreni NAN
            kruznice = self.vytvoritTFKruznici(yVoxely,rKruznice)
            koule[xR,0:konec,0:konec] = kruznice[0:konec,0:konec]
    
        print '3D objekt uspesne vytvoren'
        return koule

    def vytvoritTFKruznici(self,polomerPole,polomerKruznice):
        '''vytvori 2d pole velikosti 2xpolomerPole+1
         s kruznici o polomeru polomerKruznice uprostred '''
        radius = polomerPole
        r2 = np.arange(-radius, radius+1)**2
        dist2 = r2[:, None] + r2
        vratit =  (dist2 <= polomerKruznice**2).astype(np.int)
        return vratit
    
    def trainThresholdMap(self,data3d,voxelSize,dataShape):
        structure = self.vytvorKouli3D(voxelSize, 5)
        smoothed = ndimage.binary_opening(data3d, structure, 3)    
        spicky = smoothed != data3d
        vysledek = self.rozdelData(spicky,dataShape)
        return vysledek

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)


if __name__ == "__main__":
    main()



#! /usr/bin/python
# -*- coding: utf-8 -*-


# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../src/"))
import unittest

from PyQt4.QtGui import QFileDialog, QApplication
try:
    from pysegbase.seed_editor_qt import QTSeedEditor
except:
    logger.warning("Deprecated of pyseg_base as submodule")
    try:
        from pysegbase.seed_editor_qt import QTSeedEditor
    except:
        logger.warning("Deprecated of pyseg_base as submodule")
        from seed_editor_qt import QTSeedEditor


import numpy as np


# import pysegbase.dcmreaddata as dcmr


#  nosetests tests/organ_segmentation_test.py:OrganSegmentationTest.test_create_iparams


class QtAppTest(unittest.TestCase):
    interactiveTest = False


    @unittest.skipIf(not interactiveTest, "interactive test")
    def test_viewer_seeds(self):


        app = QApplication(sys.argv)
        dcmdir = QFileDialog.getExistingDirectory(caption='Select DICOM Folder',
                                                  options=QFileDialog.ShowDirsOnly)
        print ('ahoj')
        #app.exec_()
        #app.exit(0)

        #del(app)

        #import pdb; pdb.set_trace()
        img = np.random.rand(30,30,30)
        print '1'
        app = QApplication(sys.argv)
        print '2'
        pyed = QTSeedEditor(img)
        print '3'
        app.exec_()
        del(app)

        app = QApplication(sys.argv)
        print '2'
        pyed = QTSeedEditor(img)
        print '3'
        app.exec_()
        del(app)


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import sys
import os.path
import logging

path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../experiments/"))
# sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
# sys.path.append(os.path.join(path_to_script, "../src/"))
import unittest
from nose.plugins.attrib import attr

import experiments.tiled_liver_statistics as tls


class TextureFeaturesExperimentTest(unittest.TestCase):

    # @unittest.skip("comment after implementation")

    @attr('slow')
    def test_run_experiments(self):
        """
        """
        import lisa.texture_features as tfeat
        from sklearn import svm
        from sklearn.naive_bayes import GaussianNB
        import classification
        self.dcmdir = os.path.join(
            path_to_script, '../sample_data/jatra_06mm_jenjatraplus/')
        yaml_file = os.path.join(
            path_to_script, '../experiments/20130919_liver_statistics.yaml')

        # write_csv(fvall)
        gf = tfeat.GaborFeatures()
        glcmf = tfeat.GlcmFeatures()
        haralick = tfeat.HaralickFeatures()

        list_of_feature_fcn = [
            [tls.feat_hist, []],
            # [gf.feats_gabor, []],
            # [glcmf.feats_glcm, []],
            # [haralick.feats_haralick, [True]]
        ]
        list_of_classifiers = [
            # [GaussianNB, []],
            # [svm.SVC, []],
            [classification.GMMClassifier,
                {'n_components': 2, 'covariance_type': 'full'}],
        ]
        featrs_plus_classifs = tls.make_product_list(list_of_feature_fcn,
                                                     list_of_classifiers)

        tile_shape = [50, 50, 50]

        tls.experiment(yaml_file, yaml_file,
                       featrs_plus_classifs, tile_shape=tile_shape,
                       use_voxelsize_norm=False,
                       working_voxelsize_mm=[1, 1, 1],
                       visualization=False)


        # slab = {'none':0, 'bone':8,'lungs':9,'heart':10}
        # import pdb; pdb.set_trace()
# SupportStructureSegmentation
        # sss = support_structure_segmentation.SupportStructureSegmentation(
                # data3d = self.data3d,
                # voxelsize_mm = self.metadata['voxelsize_mm'],
                # modality = 'CT',
                # slab = slab
                #)
        # sss.lungs_segmentation()
        # sss.segmentation[260:270,160:170,1:10] = 2
        # sss.visualization()
        # total number of voxels segmented as bones in spine
        # probebox1 = sss.segmentation [260:270,160:170,1:10]== slab['lungs']
        # self.assertGreater(np.sum(probebox1),20)
        # total number of voexel segmented as none in upper left corner
        # probebox1 = sss.segmentation[10:20,10:20,5:15] == slab['none']
        # self.assertGreater(np.sum(probebox1),900)
        # import pdb; pdb.set_trace()
if __name__ == "__main__":
    logging.basicConfig(stream=sys.stderr)
    logging.getLogger().setLevel(logging.DEBUG)
    unittest.main()



# ! /usr/bin/python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)

# import funkcí z jiného adresáře
import sys
import os.path
import shutil

# path_to_script = os.path.dirname(os.path.abspath(__file__))
import unittest

import numpy as np
from nose.plugins.attrib import attr


from lisa import organ_segmentation
import lisa.dataset
import lisa.lisa_data


# nosetests tests/organ_segmentation_test.py:OrganSegmentationTest.test_create_iparams # noqa


class OrganSegmentationTest(unittest.TestCase):

    def generate_data(self):

        img3d = (np.random.rand(30, 30, 30)*10).astype(np.int16)
        seeds = (np.zeros(img3d.shape)).astype(np.int8)
        segmentation = (np.zeros(img3d.shape)).astype(np.int8)
        segmentation[10:25, 4:24, 2:16] = 1
        img3d = img3d + segmentation*20
        seeds[12:18, 9:16, 3:6] = 1
        seeds[19:22, 21:27, 19:21] = 2

        voxelsize_mm = [5, 5, 5]
        metadata = {'voxelsize_mm': voxelsize_mm}
        return img3d, metadata, seeds, segmentation

    # @unittest.skip("in progress")
    def test_sync_paul(self):
        """
        sync with paul account
        """

        path_to_paul = lisa.lisa_data.path('sync','paul')

        if os.path.exists(path_to_paul):
            shutil.rmtree(path_to_paul)
        oseg = organ_segmentation.OrganSegmentation(None)
        oseg.sync_lisa_data('paul','P4ul')


        file_path = lisa.lisa_data.path('sync','paul', 'from_server', 'test.txt')
        logger.debug('file_path %s', file_path)
        self.assertTrue(os.path.exists(file_path))

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 mjirik <mjirik@mjirik-HP-Compaq-Elite-8300-MT>
#
# Distributed under terms of the MIT license.

"""

"""
import unittest
from nose.plugins.attrib import attr
from nose.tools import nottest
import io3d
import numpy as np
import sys
import os

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../../pysegbase/src"))

from lisa import liver_segmentation_cerveny as liver_segmentation
import os
import lisa.dataset
path_to_script = os.path.dirname(os.path.abspath(__file__))
import logging
logger = logging.getLogger(__name__)

from lisa.dataset import join_sdp
from lisa import organ_segmentation

class LiverSegmentationCervenyTest(unittest.TestCase):

    # test se pouští ze složky lisa
    # nosetests tests/liver_segmentation_test.py -a actual

    @attr('interactive')
    def test_automatic(self):
        pass

    @attr('actual')
    @attr('slow')
    def test_liver_segmentation_method_3_real_data(self):
        # path_to_script = os.path.dirname(os.path.abspath(__file__))
        # dpath = os.path.join(path_to_script, '../sample_data/jatra_5mm/')
        dpath = join_sdp('liver-orig001.mhd')
        data3d, metadata = io3d.datareader.read(dpath)
        voxelsize_mm = metadata['voxelsize_mm']

        ls = liver_segmentation.LiverSegmentation(
            data3d=data3d,
            voxelsize_mm=voxelsize_mm,
            segparams={'cisloMetody': 3}
            # seeds=seeds
        )
        ls.run()
        volume = np.sum(ls.segmentation == 1) * np.prod(voxelsize_mm)

        # import sed3
        # ed = sed3.sed3(data3d, contour=ls.segmentation)  # , seeds=seeds)
        # ed.show()


        # mel by to být litr. tedy milion mm3
        self.assertGreater(volume, 100000)
        self.assertLess(volume, 2100000)

    @attr('slow')
    def test_liver_segmentation_method_4_real_data(self):
        # path_to_script = os.path.dirname(os.path.abspath(__file__))
        # dpath = os.path.join(path_to_script, '../sample_data/jatra_5mm/')
        dpath = join_sdp('liver-orig001.mhd')
        data3d, metadata = io3d.datareader.read(dpath)
        voxelsize_mm = metadata['voxelsize_mm']

        ls = liver_segmentation.LiverSegmentation(
            data3d=data3d,
            voxelsize_mm=voxelsize_mm,
            segparams={'cisloMetody': 4}
            # seeds=seeds
        )
        ls.run()
        volume = np.sum(ls.segmentation == 1) * np.prod(voxelsize_mm)

        # import sed3
        # ed = sed3.sed3(data3d, contour=ls.segmentation)  # , seeds=seeds)
        # ed.show()


        # mel by to být litr. tedy milion mm3
        self.assertGreater(volume, 100000)
        self.assertLess(volume, 2100000)

    @attr('interactive')
    def test_liver_segmentation(self):
        import numpy as np
        # import sed3
        img3d = np.random.rand(32, 64, 64) * 4
        img3d[4:24, 12:32, 5:25] = img3d[4:24, 12:32, 5:25] + 25

# seeds
        seeds = np.zeros([32, 64, 64], np.int8)
        seeds[9:12, 13:29, 18:24] = 1
        seeds[9:12, 4:9, 3:32] = 2
# [mm]  10 x 10 x 10        # voxelsize_mm = [1, 4, 3]
        voxelsize_mm = [5, 5, 5]

        ls = liver_segmentation.LiverSegmentation(
            data3d=img3d,
            voxelsize_mm=voxelsize_mm,
            # seeds=seeds
        )
        ls.run()
        volume = np.sum(ls.segmentation == 1) * np.prod(voxelsize_mm)

        # ed = sed3.sed3(img3d, contour=ls.segmentation, seeds=seeds)
        # ed.show()

        # import pdb; pdb.set_trace()

        # mel by to být litr. tedy milion mm3
        self.assertGreater(volume, 900000)
        self.assertLess(volume, 1100000)

    @attr('interactive')
    def test_liver_segmenation_just_run(self):
        """
        Tests only if it run. No strong assert.
        """
        import numpy as np
        img3d = np.random.rand(32, 64, 64) * 4
        img3d[4:24, 12:32, 5:25] = img3d[4:24, 12:32, 5:25] + 25

# seeds
        seeds = np.zeros([32, 64, 64], np.int8)
        seeds[9:12, 13:29, 18:24] = 1
        seeds[9:12, 4:9, 3:32] = 2
# [mm]  10 x 10 x 10        # voxelsize_mm = [1, 4, 3]
        voxelsize_mm = [5, 5, 5]

        ls = liver_segmentation.LiverSegmentation(
            data3d=img3d,
            voxelsize_mm=voxelsize_mm,
            # seeds=seeds
        )
        ls.run()

        # ed = sed3.sed3(img3d, contour=ls.segmentation, seeds=seeds)
        # ed.show()

    @attr('incomplete')
    @nottest
    def test_automatickyTest(self):
        ''' nacte prvni dva soubory koncici .mhd z adresare sample_data
        prvni povazuje za originalni a provede na nem segmentaci defaultni
        metodou z liver_segmentation. Pote nacte druhy a povazuje jej za
        rucni segmentaci, na vysledku a rucni provede srovnani a podle
        vysledku vypise verdikt na konzoli'''

        import io3d

        logger.setLevel(logging.DEBUG) #ZDE UPRAVIT POKUD NECHCETE VSECHNY VYPISY



        # path_to_script = os.path.dirname(os.path.abspath(__file__))
        # # print path_to_script
        # b = path_to_script[0:-5]
        # b = b + 'sample_data'
        cesta = lisa.dataset.sample_data_path()

        logger.info('probiha nacitani souboru z adresare sample_data')
        seznamSouboru = liver_segmentation.vyhledejSoubory(cesta)
        reader = io3d.DataReader()
        vektorOriginal = liver_segmentation.nactiSoubor(
            cesta, seznamSouboru, 0, reader)
        originalPole = vektorOriginal[0]
        originalVelikost = vektorOriginal[1]
        logger.info( '***zahajeni segmentace***')
        vytvoreny = liver_segmentation.LiverSegmentation(
            originalPole, originalVelikost)
        #vytvoreny.setCisloMetody(2)
        vytvoreny.run()
        segmentovany = vytvoreny.segmentation
        segmentovanyVelikost = vytvoreny.voxelSize
        logger.info('segmentace dokoncena, nacitani rucni segmentace z adresare sample_data')
        vektorOriginal = liver_segmentation.nactiSoubor(
            cesta, seznamSouboru, 1, reader)
        rucniPole = vektorOriginal[0]
        rucniVelikost = vektorOriginal[1]
        logger.info('zahajeni vyhodnoceni segmentace')
        vysledky = liver_segmentation.vyhodnoceniSnimku(
            rucniPole, rucniVelikost, segmentovany, segmentovanyVelikost)
        logger.info(str(vysledky))
        skore = vysledky[1]
        pravda = True
        import sed3
        ed = sed3.sed3(vytvoreny.data3d, contour=vytvoreny.segmentation)
        ed.show()


        if(skore > 75):
            logger.info('metoda funguje uspokojive')
            pravda = False
        if(pravda and (skore > 50)):
            logger.info('metoda funguje relativne dobre')
            pravda = False
        if(pravda):
            logger.info('metoda funguje spatne')
            pravda = False
        # self.assertGreater(skore, 5)

        self.assertLess(vysledky[0]['voe'], 50)

        return


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-
"""
Module is used for visualization of segmentation stored in pkl file.
"""

import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/dicom2fem/src"))
import logging
logger = logging.getLogger(__name__)

# from PyQt4.QtCore import Qt
from PyQt4.QtGui import QApplication
import argparse


import numpy as np
try:
    import dicom2fem
    import dicom2fem.seg2fem
    # from dicom2fem import seg2fem
    from dicom2fem.seg2fem import gen_mesh_from_voxels_mc, smooth_mesh
except:

    print('dicom2fem not found')
    # logger.warning('dicom2fem not found')
    from seg2mesh import gen_mesh_from_voxels, smooth_mesh
import misc
import viewer


def showSegmentation(
        segmentation,
        voxelsize_mm=np.ones([3, 1]),
        degrad=4,
        label=1,
        smoothing=True,
        vtk_file=None,
        qt_app=None,
        show=True,
        resize_mm=None
        ):
    """
    Funkce vrací trojrozměrné porobné jako data['segmentation']
    v data['slab'] je popsáno, co která hodnota znamená
    """

    if vtk_file is None:
        vtk_file = "mesh_geom.vtk"
    vtk_file = os.path.expanduser(vtk_file)

    labels = []

    segmentation = segmentation[::degrad, ::degrad, ::degrad]
    voxelsize_mm = voxelsize_mm * degrad

    _stats(segmentation)
    if resize_mm is not None:
        logger.debug("resize begin")
        print "resize"
        new_voxelsize_mm = np.asarray([resize_mm, resize_mm, resize_mm])
        import imtools
        segmentation = imtools.misc.resize_to_mm(segmentation, voxelsize_mm=voxelsize_mm, new_voxelsize_mm=new_voxelsize_mm)
        voxelsize_mm = new_voxelsize_mm
        logger.debug("resize begin")
    _stats(segmentation)

    # import pdb; pdb.set_trace()
    mesh_data = gen_mesh_from_voxels_mc(segmentation, voxelsize_mm)
    if smoothing:
        mesh_data.coors = smooth_mesh(mesh_data)
        # mesh_data.coors = seg2fem.smooth_mesh(mesh_data)

    else:
        mesh_data = gen_mesh_from_voxels_mc(segmentation, voxelsize_mm * 1.0e-2)
        # mesh_data.coors +=
    mesh_data.write(vtk_file)
    if qt_app is None:
        qt_app = QApplication(sys.argv)
        logger.debug("qapp constructed")
    if show:
        view = viewer.QVTKViewer(vtk_file)
        print ('show viewer')
        view.exec_()

    return labels

def _stats(data):
    print "stats"
    un = np.unique(data)
    for lab in un:
        print lab, " : ", np.sum(data==lab)

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(
        description='\
            3D visualization of segmentation\n\
            \npython show_segmentation.py\n\
            \npython show_segmentation.py -i resection.pkl -l 2 3 4 -d 4')
    parser.add_argument(
        '-i', '--inputfile',
        default='organ.pkl',
        help='input file')
    parser.add_argument(
        '-o', '--outputfile',
        default='~/lisa_data/mesh_geom.vtk',
        help='output file')
    parser.add_argument(
        '-d', '--degrad', type=int,
        default=4,
        help='data degradation, default 4')
    parser.add_argument(
        '-r', '--resize', type=float,
        default=None,
        help='resize voxel to defined size in milimeters, default is None')
    parser.add_argument(
        '-l', '--label', type=int, metavar='N', nargs='+',
        default=[1],
        help='segmentation labels, default 1')
    args = parser.parse_args()

    # data = misc.obj_from_file(args.inputfile, filetype='pickle')
    import io3d
    data = io3d.read(args.inputfile, dataplus_format=True)
    # args.label = np.array(eval(args.label))
    # print args.label
    # import pdb; pdb.set_trace()
    ds = np.zeros(data['segmentation'].shape, np.bool)
    for i in range(0, len(args.label)):
        ds = ds | (data['segmentation'] == args.label[i])

    outputfile = os.path.expanduser(args.outputfile)

    showSegmentation(ds, degrad=args.degrad, voxelsize_mm=data['voxelsize_mm'], vtk_file=outputfile, resize_mm=args.resize)

if __name__ == "__main__":
    main()



import sklearn
from sklearn import svm
import tiled_liver_statistics

from tiled_liver_statistics import feat_hist, super_feat_hist, lbp3d, f_lbp3d

def f1(im):
    return super_feat_hist(im, 4)
    

fc = [
    #[feat_hist, svm.SVC],
    #[feat_hist, sklearn.naive_bayes.GaussianNB],
    #[f1, sklearn.naive_bayes.GaussianNB],
    #[lambda im: super_feat_hist(im, 4), svm.SVC],
    #[lambda im: super_feat_hist(im, 4), sklearn.naive_bayes.GaussianNB]
    #[lambda im: f_lbp3d(im), svm.SVC],
    [lambda im: lbp3d(im, '/home/petr/Dokumenty/git/lbpLibrary/masks/mask3D_8_4.json', True), svm.SVC]
]




#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/sed3/"))
#sys.path.append(os.path.join(path_to_script, "../extern/"))
#import featurevector
import unittest

import logging
logger = logging.getLogger(__name__)


from PyQt4.QtCore import Qt
from PyQt4.QtGui import QApplication, QMainWindow, QWidget,\
        QGridLayout, QLabel, QPushButton, QFrame, QFileDialog,\
        QFont, QInputDialog, QComboBox, QRadioButton, QButtonGroup

#import apdb
#  apdb.set_trace();
#import scipy.io
import numpy as np
import scipy
#from scipy import sparse
import traceback

# ----------------- my scripts --------
import sed3
#
try:
    import dcmreaddata as dcmr
except:
    from pysegbase import dcmreaddata as dcmr
try:
    from pysegbase import pycut
except:
    logger.warning("Deprecated of pyseg_base as submodule")
    import pycut
import argparse
#import sed3

import segmentation
import qmisc
import misc
import organ_segmentation
try:
    from pysegbase import seed_editor_qt
except:
    logger.warning("Deprecated of pyseg_base as submodule")
    import seed_editor_qt



class MainWindow(QMainWindow):
    def __init__(self):
        self.inps = {'os_inps':{}}
        self.pars = {'os_pars':{}}

        pass

    def run(self, qt_app=None):
        self.qt_app = qt_app
        QMainWindow.__init__(self)


        self.initUI()


    def initUI(self):
        cw = QWidget()
        self.setCentralWidget(cw)
        grid = QGridLayout()
        grid.setSpacing(15)

        # status bar
        self.statusBar().showMessage('Ready')

        font_label = QFont()
        font_label.setBold(True)
        ################ dicom reader
        rstart = 0
        text_dcm = QLabel('DICOM reader')
        text_dcm.setFont(font_label)
        self.text_dcm_dir = QLabel('DICOM dir:')
        self.text_dcm_data = QLabel('DICOM data:')
        self.text_dcm_out = QLabel('output file:')
        grid.addWidget(text_dcm, rstart + 0, 1, 1, 4)
        grid.addWidget(self.text_dcm_dir, rstart + 1, 1, 1, 4)
        grid.addWidget(self.text_dcm_data, rstart + 2, 1, 1, 4)
        grid.addWidget(self.text_dcm_out, rstart + 3, 1, 1, 4)
        btn_dcmdir = QPushButton("Load DICOM", self)
        btn_dcmdir.clicked.connect(self.loadDcmDir)
        btn_dcmred = QPushButton("Organ Segmentation", self)
        btn_dcmred.clicked.connect(self.organSegmentation)
        btn_dcmcrop = QPushButton("Crop", self)
        btn_dcmcrop.clicked.connect(self.cropDcm)
        btn_dcmsave = QPushButton("Save DCM", self)
        btn_dcmsave.clicked.connect(self.saveDcm)
        grid.addWidget(btn_dcmdir, rstart + 4, 1)
        grid.addWidget(btn_dcmred, rstart + 4, 2)
        grid.addWidget(btn_dcmcrop, rstart + 4, 3)
        grid.addWidget(btn_dcmsave, rstart + 4, 4)

        hr = QFrame()
        hr.setFrameShape(QFrame.HLine)
        grid.addWidget(hr, rstart + 5, 0, 1, 6)

        # quit
        btn_quit = QPushButton("Quit", self)
        btn_quit.clicked.connect(self.quit)
        grid.addWidget(btn_quit, 24, 2, 1, 2)

        cw.setLayout(grid)
        self.setWindowTitle('liver-surgery')
        self.show()

    def getPars(self):
        pass

    def saveDcm(self, event=None, filename=None):
        if self.dcm_3Ddata is not None:
            self.statusBar().showMessage('Saving DICOM data...')
            QApplication.processEvents()

            if filename is None:
                filename = str(QFileDialog.getSaveFileName(self, 'Save DCM file',
                                                           filter='Files (*.dcm)'))
            if len(filename) > 0:
                savemat(filename, {'data': self.dcm_3Ddata,
                                   'voxelsize_mm': self.voxel_size_mm,
                                   'offsetmm': self.dcm_offsetmm},
                                   appendmat=False)

                self.setLabelText(self.text_dcm_out, filename)
                self.statusBar().showMessage('Ready')

            else:
                self.statusBar().showMessage('No output file specified!')

        else:
            self.statusBar().showMessage('No DICOM data!')

    def loadDcmDir(self, event=None, filename=None):
        self.statusBar().showMessage('Loading DICOM data...')
        QApplication.processEvents()


        # TODO uninteractive Serie selection



        #self.data3d, self.metadata = dcmr.dcm_read_from_dir(datadir)
        datadir = dcmr.get_dcmdir_qt(self.qt_app)

        # @TODO dialog v qt
        reader = dcmr.DicomReader(datadir)#, qt_app=self.qt_app)
        self.data3d = reader.get_3Ddata()
        self.metadata = reader.get_metaData()
        self.inps['series_number'] = reader.series_number
        self.inps['datadir'] = datadir

        self.statusBar().showMessage('DICOM data loaded')

        #QApplication.processEvents()
#        if len(filename) > 0:
#
#            data = loadmat(filename,
#                           variable_names=['data', 'voxelsize_mm', 'offsetmm'],
#                           appendmat=False)
#
#            self.dcm_3Ddata = data['data']
#            self.voxel_size_mm = data['voxelsize_mm']
#            self.dcm_offsetmm = data['offsetmm']
#            self.setVoxelVolume(self.voxel_size_mm.reshape((3,)))
#            self.setLabelText(self.text_seg_in, filename)
#            self.statusBar().showMessage('Ready')
#
#        else:
#            self.statusBar().showMessage('No input file specified!')
#    def loadDcmDir(self):
#        pass
    def reduceDcm(self):
        pass
    def cropDcm(self):
        pyed = seed_editor_qt.QTSeedEditor(self.data3d, mode='crop')
        pyed.exec_()
        pass
    def organSegmentation(self):
        self.inps['os_inps'].update({
            'data3d':self.data3d,
            'metadata':self.metadata
            })
        os_pars_all = self.pars['os_pars'].copy()
        os_pars_all.update(self.inps['os_inps'])
        os_pars_all.update({'qt_app':self.qt_app})
        print os_pars_all
        oseg = organ_segmentation.OrganSegmentation(**os_pars_all)
      #  args.dcmdir,
      #          working_voxelsize_mm=args.voxelsize_mm,
      #          manualroi=args.manualroi,
      #          texture_analysis=args.textureanalysis,
      #          edit_data=args.editdata,
      #          smoothing=args.segmentation_smoothing,
      #          iparams=args.iparams
      #          )

        oseg.interactivity()

        print (
                "Volume " +
                str(oseg.get_segmented_volume_size_mm3() / 1000000.0) + ' [l]')



    def quit(self,event):
        self.close()





def main():

    #logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(
            description='Segment vessels from liver \n\
                    \npython organ_segmentation.py\n\
                    \npython organ_segmentation.py -mroi -vs 0.6')
    parser.add_argument('-dd', '--dcmdir',
            default=None,
            help='path to data dir')
    parser.add_argument('-d', '--debug', action='store_true',
            help='run in debug mode')
    parser.add_argument('-vs', '--voxelsize_mm', default='3', type=str,
            help='Insert working voxelsize. It can be number or \
            array of three numbers. \n \
            -vs 3 \n \
            -vs [3,3,5]')
    parser.add_argument('-mroi', '--manualroi', action='store_true',
            help='manual crop before data processing')
    parser.add_argument('-iparams', '--iparams',
            default=None,
            help='filename of ipars file with stored interactivity')
    parser.add_argument('-t', '--tests', action='store_true',
            help='run unittest')
    parser.add_argument('-tx', '--textureanalysis', action='store_true',
            help='run with texture analysis')
    parser.add_argument('-exd', '--exampledata', action='store_true',
            help='run unittest')
    parser.add_argument('-ed', '--editdata', action='store_true',
            help='Run data editor')
    parser.add_argument('-so', '--show_output', action='store_true',
            help='Show output data in viewer')
    parser.add_argument(
            '-ss',
            '--segmentation_smoothing',
            action='store_true',
            help='Smoothing of output segmentation',
            default=False
            )
    args = parser.parse_args()

    # voxelsize_mm can be number or array
    args.voxelsize_mm = np.array(eval(args.voxelsize_mm))

    if args.debug:
        logger.setLevel(logging.DEBUG)

    if args.tests:
        # hack for use argparse and unittest in one module
        sys.argv[1:] = []
        unittest.main()
        sys.exit()

    if args.exampledata:

        args.dcmdir = '../sample_data/\
                matlab/examples/sample_data/DICOM/digest_article/'

    if args.iparams is not None:
        args.iparams = misc.obj_from_file(args.iparams)
    #else:
    #dcm_read_from_dir('/home/mjirik/data/medical/data_orig/46328096/')
        #data3d, metadata = dcmreaddata.dcm_read_from_dir()

    oseg = OrganSegmentation(args.dcmdir,
            working_voxelsize_mm=args.voxelsize_mm,
            manualroi=args.manualroi,
            texture_analysis=args.textureanalysis,
            edit_data=args.editdata,
            smoothing=args.segmentation_smoothing,
            iparams=args.iparams
            )

    oseg.interactivity()

    #igc = pycut.ImageGraphCut(data3d, zoom = 0.5)
    #igc.interactivity()

    #igc.make_gc()
    #igc.show_segmentation()

    # volume
    #volume_mm3 = np.sum(oseg.segmentation > 0) * np.prod(oseg.voxelsize_mm)

    print (
            "Volume " +
            str(oseg.get_segmented_volume_size_mm3() / 1000000.0) + ' [l]')
    #pyed = sed3.sed3(oseg.data3d, contour =
    # oseg.segmentation)
    #pyed.show()

    if args.show_output:
        oseg.show_output()

    savestring = raw_input('Save output data? (y/n): ')
    #sn = int(snstring)
    if savestring in ['Y', 'y']:

        data = oseg.export()

        misc.obj_to_file(data, "organ.pkl", filetype='pickle')
        misc.obj_to_file(oseg.get_ipars(), 'ipars.pkl', filetype='pickle')
    #output = segmentation.vesselSegmentation(oseg.data3d,
    # oseg.orig_segmentation)


    def checkSegData(self):
        if self.segmentation_data is None:
            self.statusBar().showMessage('No SEG data!')
            return
        nzs = self.segmentation_data.nonzero()
        nn = nzs[0].shape[0]
        if nn > 0:
            aux = ' voxels = %d, volume = %.2e mm3' % (nn, nn * self.voxel_volume)
            self.setLabelText(self.text_seg_data, aux)
            self.setLabelText(self.text_mesh_in, 'segmentation data')
            self.statusBar().showMessage('Ready')

        else:
            self.statusBar().showMessage('Zero SEG data!')

    def autoSeg(self):
        if self.dcm_3Ddata is None:
            self.statusBar().showMessage('No DICOM data!')
            return
        igc = pycut.ImageGraphCut(self.dcm_3Ddata,
                voxelsize=self.voxel_size_mm)

        pyed = QTSeedEditor(self.dcm_3Ddata,
                seeds=self.segmentation_seeds,
                modeFun=igc.interactivity_loop,
                voxelVolume=self.voxel_volume)
        pyed.exec_()

        self.segmentation_data = pyed.getContours()
        self.segmentation_seeds = pyed.getSeeds()
        self.checkSegData()



def main2():
    app = QApplication(sys.argv)
    mw = MainWindow()
    mw.run(qt_app=app)

    sys.exit(app.exec_())


if __name__ == "__main__":
    main2()



#! /usr/bin/python
# -*- coding: utf-8 -*-

# import funkcí z jiného adresáře
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
import unittest

import numpy as np


#
import pysegbase.dcmreaddata as dcmr
import lisa.support_structure_segmentation


class SupportStructureSegmentationTest(unittest.TestCase):

    def setUp(self):
        # self.dcmdir = os.path.join(
        #     path_to_script, '../sample_data/jatra_06mm_jenjatraplus/')
        self.dcmdir = os.path.join(path_to_script, '../sample_data/jatra_5mm')
        # self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
        reader = dcmr.DicomReader(self.dcmdir)
        self.data3d = reader.get_3Ddata()
        self.metadata = reader.get_metaData()

    @unittest.skip("comment after implementation")
    def test_bone_segmentation(self):
        """
        Check values in two areas.

        Area1: total number of voxels segmented as bones in probebox in spine.

        Area2: total number of voexel segmented as none in upper left corner.
        """

        slab = {'none': 0, 'bone': 8, 'lungs': 9, 'heart': 10}
        # import pdb; pdb.set_trace()
#            SupportStructureSegmentation
        sss = lisa.support_structure_segmentation.SupportStructureSegmentation(
            data3d=self.data3d,
            voxelsize_mm=self.metadata['voxelsize_mm'],
            modality='CT',
            slab=slab

        )

        sss.bone_segmentation()

        # total number of voxels segmented as bones in spine
        probebox1 = sss.segmentation[360:370, 260:270, 5:15] == slab['bone']
        self.assertGreater(np.sum(probebox1), 20)

        # total number of voexel segmented as none in upper left corner
        probebox1 = sss.segmentation[10:20, 10:20, 5:15] == slab['none']
        self.assertGreater(np.sum(probebox1), 900)

        # sss.visualization()
        # import pdb; pdb.set_trace()

    @unittest.skip("comment after implementation")
    def test_lungs_segmentation(self):
        """
        Check values in two areas.

        Area1: total number of voxels segmented as bones in probebox in lungs.

        Area2: total number of voexel segmented as none in upper left corner.
        """

        slab = {'none': 0, 'bone': 8, 'lungs': 9, 'heart': 10}
        # import pdb; pdb.set_trace()
#            SupportStructureSegmentation
        sss = lisa.support_structure_segmentation.SupportStructureSegmentation(
            data3d=self.data3d,
            voxelsize_mm=self.metadata['voxelsize_mm'],
            modality='CT',
            slab=slab

        )

        sss.lungs_segmentation()
        # sss.segmentation[260:270,160:170,1:10] = 2
        # sss.visualization()
        # total number of voxels segmented as bones in spine
        probebox1 = sss.segmentation[260:270, 160:170, 1:10] == slab['lungs']
        self.assertGreater(np.sum(probebox1), 20)

        # total number of voexel segmented as none in upper left corner
        probebox1 = sss.segmentation[10:20, 10:20, 5:15] == slab['none']
        self.assertGreater(np.sum(probebox1), 900)

        # import pdb; pdb.set_trace()


#    def test_synthetic_data_lesions_automatic_localization(self):
#        """
#        Function uses lesions  automatic localization in synthetic data.
#        """
# dcmdir = os.path.join(
#   path_to_script,
#   './../sample_data/matlab/examples/sample_data/DICOM/digest_article/')
# data
#        slab = {'none':0, 'liver':1, 'porta':2, 'lesions':6}
#        voxelsize_mm = np.array([1.0,1.0,1.2])
#
#        segm = np.zeros([256,256,80], dtype=np.int16)
#
# liver
#        segm[70:190,40:220,30:60] = slab['liver']
# port
#        segm[120:130,70:220,40:45] = slab['porta']
#        segm[80:130,100:110,40:45] = slab['porta']
#        segm[120:170,130:135,40:44] = slab['porta']
#
# vytvoření kopie segmentace - před určením lézí
#        segm_pre = copy.copy(segm)
#
#        segm[150:180,70:105,42:55] = slab['lesions']
#
#
#        data3d = np.zeros(segm.shape)
#        data3d[segm== slab['none']] = 680
#        data3d[segm== slab['liver']] = 1180
#        data3d[segm== slab['porta']] = 1230
#        data3d[segm== slab['lesions']] = 1110
# noise = (np.random.rand(segm.shape[0], segm.shape[1],
#                         segm.shape[2])*30).astype(np.int16)
# noise = (np.random.normal(0,30,segm.shape))#.astype(np.int16)
#        data3d = (data3d + noise  ).astype(np.int16)
#
#
#        data={'data3d':data3d,
#                'slab':slab,
#                'voxelsize_mm':voxelsize_mm,
#                'segmentation':segm_pre
#                }
#
# @TODO je tam bug, prohlížeč neumí korektně pracovat s doubly
# app = QApplication(sys.argv)
# pyed = QTSeedEditor(noise )
# pyed = QTSeedEditor(data3d)
# pyed.exec_()
# img3d = np.zeros([256,256,80], dtype=np.int16)
#
# pyed = sed3.sed3(data3d)
# pyed.show()
#
#        tumory = lesions.Lesions()
#
#        tumory.import_data(data)
#        tumory.automatic_localization()
# tumory.visualization()
#
#
#
# ověření výsledku
# pyed = sed3.sed3(outputTmp, contour=segm==slab['porta'])
# pyed.show()
#
#        errim = np.abs(
#                (tumory.segmentation == slab['lesions']).astype(np.int) -
#                (segm == slab['lesions']).astype(np.int))
#
# ověření výsledku
# pyed = sed3.sed3(errim, contour=segm==slab['porta'])
# pyed.show()
# evaluation
#        sum_of_wrong_voxels = np.sum(errim)
#        sum_of_voxels = np.prod(segm.shape)
#
# print "wrong ", sum_of_wrong_voxels
# print "voxels", sum_of_voxels
#
#        errorrate = sum_of_wrong_voxels/sum_of_voxels
#
#
#        self.assertLess(errorrate,0.1)
#
#
#
if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-



# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../src/"))
import unittest

import numpy as np


import organ_segmentation
import pysegbase.dcmreaddata as dcmr


#  nosetests tests/organ_segmentation_test.py:OrganSegmentationTest.test_create_iparams


class TexturesTest(unittest.TestCase):
    interactiveTest = False
    verbose = False

    def test_texture_features(self):
        """
        Interactivity is stored to file
        """
        try:
            from pysegbase.seed_editor_qt import QTSeedEditor
        except:
            logger.warning("Deprecated of pyseg_base as submodule")
            from seed_editor_qt import QTSeedEditor
        from PyQt4.QtGui import QApplication
        from skimage.feature import greycomatrix, greycoprops
        import misc
        dcmdir = os.path.join(path_to_script,'./../sample_data/jatra_5mm')
        
        #gcparams = {'pairwiseAlpha':10, 'use_boundary_penalties':True}
        #segparams = {'pairwise_alpha_per':3, 'use_boundary_penalties':True,'boundary_penalties_sigma':200}
        #oseg = organ_segmentation.OrganSegmentation(dcmdir, working_voxelsize_mm = 4, segparams=segparams)
        #oseg.add_seeds_mm([120],[120],[70], label=1, radius=30)
        #oseg.add_seeds_mm([170,220,250],[250,280,200],[70], label=2, radius=30)


        reader = dcmr.DicomReader(dcmdir) # , qt_app=qt_app)
        data3d = reader.get_3Ddata()
# normalizace na čísla od nuly do 255
        data3d = data3d/(2**4)
        metadata = reader.get_metaData()
        iparams = {}
        iparams['series_number'] = reader.series_number
        iparams['datadir'] = dcmdir

        working_voxelsize_mm = 2

        voxelsize_mm = np.array(metadata['voxelsize_mm'])
        zoom = voxelsize_mm / working_voxelsize_mm

        PATCH_SIZE = 21
        PATCH_DIST = 15
        shp = data3d.shape
        vx, vy, vz = np.mgrid[0:shp[0] - PATCH_SIZE:PATCH_DIST, 
                0:shp[1] - PATCH_SIZE:PATCH_DIST,
                0:shp[2] - PATCH_SIZE:8]

        
        import pdb; pdb.set_trace()

        feat = np.zeros(vx.shape)
        feat2 = np.zeros(vx.shape)

        #vx = vx.reshape(-1)
        #vy = vy.reshape(-1)
        #vz = vz.reshape(-1)


        

        #for i in range(0,len(vx)):
        it = np.nditer(vx, flags=['multi_index'])
        while not it.finished:
            vxi = vx[it.multi_index]
            vyi = vy[it.multi_index]
            vzi = vz[it.multi_index]

            

            patch = data3d[
                vxi:vxi + PATCH_SIZE,
                vyi:vyi + PATCH_SIZE,
                vzi
                ]
            patch = np.squeeze(patch)
            print  it.iterindex, ' - ', vxi, ' ',  vyi, ' ',vzi , ' - ',it.multi_index
            #import pdb; pdb.set_trace()
            glcm = greycomatrix(patch,
                [5],[0],256,
                symmetric=True, normed=True)
            dissimilarity = greycoprops(glcm, 'dissimilarity')
            feat[it.multi_index] = dissimilarity
            feat2[it.multi_index] = greycoprops(glcm,'correlation')
            it.iternext()





        locations = [(474, 291), (440, 433), (466, 18), (462, 236)]




        qt_app = QApplication(sys.argv)
        pyed = QTSeedEditor(feat)
        qt_app.exec_()
        pyed = QTSeedEditor(feat2)
        qt_app.exec_()

        pdb


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

"""
Generator of histology report

"""
import logging
logger = logging.getLogger(__name__)

import sys
import os.path
path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/dicom2fem/src"))


import argparse
import numpy as np
import scipy.ndimage
from io3d import datawriter
import misc

# import datareader
import sed3 as se

import gen_vtk_tree

import vtk
from vtk.util import numpy_support

from datetime import datetime


class VolumeTreeGenerator:
    """
    This generator is called by generateTree() function as a general form.
    Other similar generator is used for generating LAR outputs.
    """
    def __init__(self, gtree):
        self.shape = gtree.shape
        self.data3d = np.zeros(gtree.shape, dtype=np.int)
        self.voxelsize_mm = gtree.voxelsize_mm

    def add_cylinder(self, p1m, p2m, rad, id):
        """
        Funkce na vykresleni jednoho segmentu do 3D dat
        """

        cyl_data3d = np.ones(self.shape, dtype=np.bool)
        # prvni a koncovy bod, ve pixelech
        p1 = [p1m[0] / self.voxelsize_mm[0], p1m[1] /
              self.voxelsize_mm[1], p1m[2] / self.voxelsize_mm[2]]
        p2 = [p2m[0] / self.voxelsize_mm[0], p2m[1] /
              self.voxelsize_mm[1], p2m[2] / self.voxelsize_mm[2]]
        logger.debug(
            "p1_px: " + str(p1[0]) + " " + str(p1[1]) + " " + str(p1[2]))
        logger.debug(
            "p2_px: " + str(p2[0]) + " " + str(p2[1]) + " " + str(p2[2]))
        logger.debug("radius_mm:" + str(rad))

        # vzdalenosti mezi prvnim a koncovim bodem (pro jednotlive osy)
        pdiff = [abs(p1[0] - p2[0]), abs(p1[1] - p2[1]), abs(p1[2] - p2[2])]

        # generovani hodnot pro osu segmentu
        num_points = max(pdiff) * \
            2  # na jeden "pixel nejdelsi osy" je 2 bodu primky (shannon)
        zvalues = np.linspace(p1[0], p2[0], num_points)
        yvalues = np.linspace(p1[1], p2[1], num_points)
        xvalues = np.linspace(p1[2], p2[2], num_points)

        # drawing a line
        for i in range(0, len(xvalues)):
            try:
                cyl_data3d[int(zvalues[i])][int(yvalues[i])][int(xvalues[i])] = 0
            except:
                import traceback
                traceback.print_exc()
                print "except in drawing line"
                import ipdb; ipdb.set_trace() #  noqa BREAKPOINT

        # cuting size of 3d space needed for calculating distances (smaller ==
        # a lot faster)
        cut_up = max(
            0, round(min(p1[0], p2[0]) - (rad / min(self.voxelsize_mm)) - 2))
        # ta 2 je kuli tomu abyh omylem nurizl
        cut_down = min(self.shape[0], round(
            max(p1[0], p2[0]) + (rad / min(self.voxelsize_mm)) + 2))
        cut_yu = max(
            0, round(min(p1[1], p2[1]) - (rad / min(self.voxelsize_mm)) - 2))
        cut_yd = min(self.shape[1], round(
            max(p1[1], p2[1]) + (rad / min(self.voxelsize_mm)) + 2))
        cut_xl = max(
            0, round(min(p1[2], p2[2]) - (rad / min(self.voxelsize_mm)) - 2))
        cut_xr = min(self.shape[2], round(
            max(p1[2], p2[2]) + (rad / min(self.voxelsize_mm)) + 2))
        logger.debug("cutter_px: z_up-" + str(cut_up) + " z_down-" + str(cut_down) + " y_up-" + str(
            cut_yu) + " y_down-" + str(cut_yd) + " x_left-" + str(cut_xl) + " x_right-" + str(cut_xr))
        cyl_data3d_cut = cyl_data3d[
            int(cut_up):int(cut_down),
            int(cut_yu):int(cut_yd),
            int(cut_xl):int(cut_xr)]

        # calculating distances
        # spotrebovava naprostou vetsinu casu (pro 200^3  je to kolem 1.2
        # sekundy, proto jsou data osekana)
        lineDst = scipy.ndimage.distance_transform_edt(
            cyl_data3d_cut, self.voxelsize_mm)

        # zkopirovani vyrezu zpet do celeho rozsahu dat
        for z in xrange(0, len(cyl_data3d_cut)):
            for y in xrange(0, len(cyl_data3d_cut[z])):
                for x in xrange(0, len(cyl_data3d_cut[z][y])):
                    if lineDst[z][y][x] <= rad:
                        iX = int(z + cut_up)
                        iY = int(y + cut_yu)
                        iZ = int(x + cut_xl)
                        self.data3d[iX][iY][iZ] = 1

    def get_output(self):
        return self.data3d

    def save(self, outputfile, filetype='pklz'):
        data = {
            'data3d': self.data3d,
            'voxelsize_mm': self.voxelsize_mm
        }

        misc.obj_to_file(data, outputfile, filetype=filetype)
        print "saved"
        #dw = datawriter.DataWriter()
        #dw.Write3DData(self.data3d, outputfile, filetype)

    def show(self):
        pyed = se.sed3(self.data3d)
        pyed.show()


class TreeGenerator:

    def __init__(self, generator_class=VolumeTreeGenerator, generator_params=None):
        self.rawdata = None
        self.tree_data = None
        self.data3d = None
        self.voxelsize_mm = [1, 1, 1]
        self.shape = None
        self.use_lar = False
        self.generator_class = generator_class
        self.generator_params = generator_params

    def importFromYaml(self, filename):
        rawdata = misc.obj_from_file(filename=filename, filetype='yaml')
        self.rawdata = rawdata

        try:
            # key is usually "porta" or "microstructure"
            keys = self.rawdata['graph'].keys()
            self.tree_data = self.rawdata['graph'][keys[0]]
        except:
            self.tree_data = self.rawdata['Graph']

    def generateTree(self):
        """
        | Funkce na vygenerování objemu stromu ze zadaných dat.
        | Generates output by defined generator. If VolumeTreeGenerator is used, output is data3d.
        """
# LAR init
        if self.use_lar:
            import lar_vessels
            self.lv = lar_vessels.LarVessels()

        # use generator init
        if self.generator_params is None:
            self.generator = self.generator_class(self)
        else:
            self.generator = self.generator_class(self, **generator_params)

        for cyl_id in self.tree_data:
            logger.debug("CylinderId: " + str(cyl_id))
            cyl_data = self.tree_data[cyl_id]

            # try:
            #     cyl_data = self.data['graph']['porta'][cyl_id]
            # except:
            #     cyl_data = self.data['Graph'][cyl_id]

            # prvni a koncovy bod, v mm + radius v mm
            try:
                p1m = cyl_data['nodeA_ZYX_mm']  # souradnice ulozeny [Z,Y,X]
                p2m = cyl_data['nodeB_ZYX_mm']
                rad = cyl_data['radius_mm']
                self.generator.add_cylinder(p1m, p2m, rad, cyl_id)
            except Exception, e:
                # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT

                logger.error(
                    "Segment id " + str(cyl_id) + ": error reading data from yaml!: "+str(e))
                # return

            # if self.use_lar:
            #     self.generator.add_cylinder(p1m, p2m, rad, in)
        logger.debug("cylinders generated")

        try:
            # generator could have finish() function
            self.generator.finish()
            logger.debug("joints generated")
        except:
            import traceback
            logger.debug(traceback.format_exc())
            pass

        output = self.generator.get_output()

        logger.debug("before visualization - generateTree()")
        if self.use_lar:
            self.lv.show()
        return output

    def generateTree_vtk(self):
        """
        | Funkce na vygenerování objemu stromu ze zadaných dat.
        | Veze pro generování pomocí VTK
        | !!! funguje špatně -> vstupní data musí být pouze povrchové body, jinak generuje ve výstupních datech dutiny

        """
        # get vtkPolyData
        tree_data = gen_vtk_tree.process_tree(self.rawdata['Graph'])
        polyData = gen_vtk_tree.gen_tree(tree_data)

        polyData.GetBounds()
        # bounds = polyData.GetBounds()

        white_image = vtk.vtkImageData()
        white_image.SetSpacing(self.voxelsize_mm)
        white_image.SetDimensions(self.shape)
        white_image.SetExtent(
            [0, self.shape[0] - 1, 0, self.shape[1] - 1, 0, self.shape[2] - 1])
        # origin = [(bounds[0] + self.shape[0])/2, (bounds[1] + self.shape[1])/2, (bounds[2] + self.shape[2])/2]
        # white_image.SetOrigin(origin) #neni potreba?
        # white_image.SetScalarTypeToUnsignedChar()
        white_image.AllocateScalars()

        # fill the image with foreground voxels: (still black until stecil)
        inval = 255
        outval = 0
        count = white_image.GetNumberOfPoints()
        for i in range(0, count):
            white_image.GetPointData().GetScalars().SetTuple1(i, inval)

        pol2stencil = vtk.vtkPolyDataToImageStencil()
        pol2stencil.SetInput(polyData)

        # pol2stencil.SetOutputOrigin(origin) # TOHLE BLBNE
        pol2stencil.SetOutputSpacing(self.voxelsize_mm)
        pol2stencil.SetOutputWholeExtent(white_image.GetExtent())
        pol2stencil.Update()

        imgstenc = vtk.vtkImageStencil()
        imgstenc.SetInput(white_image)
        imgstenc.SetStencil(pol2stencil.GetOutput())
        imgstenc.ReverseStencilOff()
        imgstenc.SetBackgroundValue(outval)
        imgstenc.Update()

        # VTK -> Numpy
        vtk_img_data = imgstenc.GetOutput()
        vtk_data = vtk_img_data.GetPointData().GetScalars()
        numpy_data = numpy_support.vtk_to_numpy(vtk_data)
        numpy_data = numpy_data.reshape(
            self.shape[0], self.shape[1], self.shape[2])
        numpy_data = numpy_data.transpose(2, 1, 0)

        self.data3d = numpy_data

    def saveToFile(self, outputfile, filetype):
        self.generator.save(outputfile, filetype)

    def show(self):
        self.generator.show()


if __name__ == "__main__":
    logging.basicConfig()
    logger = logging.getLogger()
    logger.setLevel(logging.WARNING)

    # input parser
    parser = argparse.ArgumentParser(
        description='Histology analyser reporter. Try: \
python src/gen_volume_tree.py -i ./tests/hist_stats_test.yaml'
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file, yaml file'
    )
    parser.add_argument(
        '-o', '--outputfile',
        default=None,
        help='output file, .raw, .dcm, .tiff, given by extension '
    )
    parser.add_argument(
        '-ot', '--outputfiletype',
        default='pkl',
        help='output file type.  raw, dcm, tiff, or pkl,   default is pkl, '
    )
    parser.add_argument(
        '-vs', '--voxelsize',
        default=[1.0, 1.0, 1.0],
        type=float,
        metavar='N',
        nargs='+',
        help='size of voxel (ZYX)'
    )
    parser.add_argument(
        '-ds', '--datashape',
        default=[200, 200, 200],
        type=int,
        metavar='N',
        nargs='+',
        help='size of output data in pixels for each axis (ZYX)'
    )
    parser.add_argument(
        '-g', '--generator',
        default='vol',
        type=str,
        help='Volume or surface model can be generated by use this option. \
                Use "vol", "volume" for volumetric model. For LAR surface model\
                use "lar".'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    parser.add_argument(
        '-l', '--useLar', action='store_true',
        help='Use LAR')
    args = parser.parse_args()

    if args.debug:
        logger.setLevel(logging.DEBUG)

    startTime = datetime.now()

    generator_params = None
    if args.generator in ['vol', 'volume']:
        generator_class = VolumeTreeGenerator
    elif args.generator in ['lar']:
        import gt_lar
        generator_class = gt_lar.GTLar
    elif args.generator in ['kunes']:
        import gt_lar_kunes
        generator_class = gt_lar_kunes.GTLar
    elif args.generator in ['larsm']:
        import gt_lar_smooth
        generator_class = gt_lar_smooth.GTLarSmooth
    elif args.generator in ['lar_nojoints']:
        import gt_lar
        generator_class = gt_lar.GTLar
        generator_params = {
            'endDistMultiplicator': 0,
            'use_joints': False
        }

    tg = TreeGenerator(generator_class, generator_params)
    tg.importFromYaml(args.inputfile)
    tg.voxelsize_mm = args.voxelsize
    tg.shape = args.datashape
    tg.use_lar = args.useLar
    data3d = tg.generateTree()

    logger.info("TimeUsed:" + str(datetime.now() - startTime))
    # volume_px = sum(sum(sum(data3d)))
    # volume_mm3 = volume_px * \
    #     (tg.voxelsize_mm[0] * tg.voxelsize_mm[1] * tg.voxelsize_mm[2])
    # logger.info("Volume px:" + str(volume_px))
    # logger.info("Volume mm3:" + str(volume_mm3))

# vizualizace
    logger.debug("before visualization")
    tg.show()
    logger.debug("after visualization")

# ukládání do souboru
    if args.outputfile is not None:
        tg.saveToFile(args.outputfile, args.outputfiletype)



#! /usr/bin/python
# -*- coding: utf-8 -*-

from larcc import *
from lar2psm import *
from mapper import *
from splines import *


def TRIANGULAR_COONS_PATCH(params):
    ab0Fn = params[0]
    bc1Fn = params[1] 
    ca0Fn = params[2]

    def TRIANGULAR_COONS_PATCH0(point):
        u = point[0]
        v = point[1]
        w = point[2]


        if(hasattr(ab0Fn, '__call__')):
            Sab0 = ab0Fn(point)
        else:
            Sab0 = ab0Fn
        if(hasattr(bc1Fn, '__call__')):
            Sbc1 = bc1Fn(point)
        else:
            Sbc1 = bc1Fn
        if(hasattr(ca0Fn, '__call__')):
            Sca0 = ca0Fn(point)
        else:
            Sca0 = ca0Fn

        rn = len(Sab0)
        mapped = [0] * rn

        if (u == 1):
            ur = 0
        else:
            ur = ((u * v) / double(v + w))
        if (v == 1):
            vs = 0
        else:
            vs = ((v * w) / double(w + u))
        if (w == 1):
            wt = 0
        else:
            wt = ((w * u) / double(v + u))

        for i in range(0, rn):
            mapped[i] = \
                u * Sab0[i] + ur * (Sca0[i] - Sab0[i]) + \
                v * Sca0[i] + vs * (Sbc1[i] - Sca0[i]) + \
                w * Sbc1[i] + wt * (Sab0[i] - Sbc1[i])
        return mapped
    return TRIANGULAR_COONS_PATCH0


def TRIANGLE_DOMAIN(n, points):
    pa = points[0]
    pb = points[1]
    pc = points[2]
    net = []
    cells = []

    for i in range(0, n + 1):
        net.append([
            (pa[0] + i * (pb[0] - pa[0]) / double(n)),
            (pa[1] + i * (pb[1] - pa[1]) / double(n)),
            (pa[2] + i * (pb[2] - pa[2]) / double(n))
        ])

    for y in range(1, n + 1):
        r0 = (y - 1) * (n + 2) - (y - 1) * y / 2
        r1 = y * (n + 2) - y * (y + 1) / 2
        for x in range(0, n - y +1):
            c0 = r0 + x
            c1 = r1 + x
            net.append([
                pa[0] + 
                x * (pb[0] - pa[0]) / double(n) +
                y * (pc[0] - pa[0]) / double(n),
                pa[1] + 
                x * (pb[1] - pa[1]) / double(n) + 
                y * (pc[1] - pa[1]) / double(n),
                pa[2] + 
                x * (pb[2] - pa[2]) / double(n) + 
                y * (pc[2] - pa[2]) / double(n)
            ])
            if (x > 0):
                cells.append([c1, c0, c1 - 1])

            cells.append([c1, c0 + 1, c0])

        #print net[:2]
        #print cells[:2]
    return MKPOLS([net, cells])



#! /usr/bin/python
# -*- coding: utf-8 -*-
import sys
import os
path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src/"))
sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script, "../src/"))
import numpy as np

import organ_segmentation
import pysegbase.dcmreaddata as dcmr
import os.path


def main():

    dcmdir = os.path.join(path_to_script,'./../sample_data/matlab/examples/sample_data/DICOM/digest_article/')
    #data3d, metadata = dcmr.dcm_read_from_dir(dcmdir)
    oseg = organ_segmentation.OrganSegmentation(dcmdir, working_voxelsize_mm = 4, manualroi=False)

    oseg.interactivity()

main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

CONFIG_DEFAULT={
    "autocrop": False,
    "autocrop_margin_mm": [10, 10, 10],
    "data3d": None,
    "datapath": None,
    "edit_data": False,
    "manualroi": True,
    "metadata": None,
    "output_label": 1,
    "qt_app": None,
    "roi": None,
    "seeds": None,
    "segmentation_smoothing": False,
    "segmodelparams": {
        # 'mdl_stored_file': '~/lisa_data/liver_intensity.Model.p',
        # "fv_type": 'fv_extern',
        # 'fv_extern':'intensity_localization_fv',
        # 'method': 'multiscale_graphcut',
        "params": {"covariance_type": "full", "n_components": 3}
    },
    "type": "gmmsame",
    "segparams": {"pairwise_alpha_per_mm2":45, "return_only_object_with_seeds": True},
    "series_number": None,
    "slab": {},
    "smoothing_mm": 4,
    "texture_analysis": None,
    "working_voxelsize_mm": 2.0,
    "viewermax": 225,
    "viewermin": -125,
    'segmentation_alternative_params':{
        "simple 2 mm":{
            "segmodelparams": {
                'mdl_stored_file': None,
                'fv_type': 'intensity',
                'fv_extern': None,
                # 'mdl_stored_file': '~/lisa_data/liver_intensity.Model.p',
                # "fv_type": 'fv_extern',
                # 'fv_extern':'intensity_localization_fv',
                # 'method': 'multiscale_graphcut',
                "params": {"covariance_type": "full", "n_components": 3}
            },
            'working_voxelsize_mm': 2
        },
        "simple 2.5 mm":{
            "segmodelparams": {
                'mdl_stored_file': None,
                'fv_type': 'intensity',
                'fv_extern': None,
                # 'mdl_stored_file': '~/lisa_data/liver_intensity.Model.p',
                # "fv_type": 'fv_extern',
                # 'fv_extern':'intensity_localization_fv',
                # 'method': 'multiscale_graphcut',
                "params": {"covariance_type": "full", "n_components": 3}
            },
            'working_voxelsize_mm': 2.5
        },
        "label kidney L":{
            "output_label": 15,
            'clean_seeds_after_update_parameters': True,
        },
        "label kidney R":{
            "output_label": 16,
            'clean_seeds_after_update_parameters': True,
        },
        "label liver":{
            "output_label": 1,
            'clean_seeds_after_update_parameters': True,
        },
        "label hearth":{
            "output_label": 10,
            'clean_seeds_after_update_parameters': True,
        }
    }

}


#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 mjirik
#
# Distributed under terms of the MIT license.

"""

"""
import unittest
from nose.plugins.attrib import attr
import sys

from PyQt4.QtGui import QApplication
from PyQt4.QtTest import QTest
from PyQt4.QtCore import Qt
import lisa.lisaWindow
import lisa.organ_segmentation
from lisa.lisaWindow import OrganSegmentationWindow


class LisaGUITest(unittest.TestCase):

    def setUp(self):
        '''Create the GUI'''
        self.app = QApplication(sys.argv)
        oseg = lisa.organ_segmentation.OrganSegmentation()
        self.oseg_w = OrganSegmentationWindow(oseg) # noqa
        # self.form = MargaritaMixer.MargaritaMixer()

    @attr('interactive')
    def test_click(self):
        # i5 = self.oseg_w.grid.itemAt(5)
        dcmdirw = self.oseg_w.uiw['dcmdir']
        QTest.mouseClick(dcmdirw, Qt.LeftButton)

    def test_quit_gui(self):
        """
        Tests event of quit
        """
        self.assertTrue(self.oseg_w.quit(event=None))


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import numpy as np
from nose.plugins.attrib import attr
import lisa.convert
import os.path as op
import os


class MyTestCase(unittest.TestCase):
    @attr('interactive')
    def test_something(self):
        seg = np.zeros([20,21,22], dtype=np.uint8)
        seg [4:11, 3:15, 11:17] = 1
        seg [4:7, 3:10, 11:14] = 0

        path_to_script = op.dirname(op.abspath(__file__))
        fn_stl = op.join(path_to_script, 'test_stl.stl')
        fn_tmp = op.join(path_to_script, 'test_stl_tmp.vtk')

        lisa.convert.seg2stl(seg, outputfile=fn_stl, tempfile=fn_tmp)
        self.assertTrue(op.exists(fn_stl))
        os.remove(fn_stl)
        os.remove(fn_tmp)






if __name__ == '__main__':
    unittest.main()
